@@ -152,6 +152,9 @@
 
 /* Build with dmalloc support? */
 #undef DMALLOC
+ 
+/* Define if you have the getmntent() function */
+#undef HAVE_GETMNTENT
 
 ^L
 /* Leave that blank line there!!  Autoheader needs it.
