@@ -12,10 +12,10 @@
  *
  */
 
-#include <zlib.h>
 #include <sys/types.h>
 
 #include <rpmio_internal.h>
+#include <rpmlib.h>
 
 /** \ingroup payload
  * @note CPIO_CHECK_ERRNO bit is set only if errno is valid.
