@@ -9,6 +9,7 @@
 #include "system.h"
 
 #include <rpmlib.h>
+#include <rpmmacro.h>		/* XXX for rpmExpand() */
 
 #include "depends.h"
 #include "rpmdb.h"		/* XXX response cache needs dbiOpen et al. */
@@ -169,7 +170,7 @@ static void alFree(availableList al)
     if ((p = al->list) != NULL)
     for (i = 0; i < al->size; i++, p++) {
 
-	{   struct tsortInfo * tsi;
+	{   tsortInfo tsi;
 	    while ((tsi = p->tsi.tsi_next) != NULL) {
 		p->tsi.tsi_next = tsi->tsi_next;
 		tsi->tsi_next = NULL;
@@ -521,8 +522,8 @@ static void parseEVR(char * evr,
     if (rp) *rp = release;
 }
 
-/*@observer@*/ const char *rpmNAME = PACKAGE;
-/*@observer@*/ const char *rpmEVR = VERSION;
+const char *rpmNAME = PACKAGE;
+const char *rpmEVR = VERSION;
 int rpmFLAGS = RPMSENSE_EQUAL;
 
 int rpmRangesOverlap(const char * AName, const char * AEVR, int AFlags,
@@ -1122,7 +1123,7 @@ alAllSatisfiesDepend(const availableList al,
 		rc = rpmRangesOverlap(p->provides[i], proEVR, proFlags,
 				keyName, keyEVR, keyFlags);
 		if (rc)
-		    /*@innerbreak@*/ break;
+		    /*@switchbreak@*/ break;
 	    }
 	    if (keyType && keyDepend && rc)
 		rpmMessage(RPMMESS_DEBUG, _("%s: %-45s YES (added provide)\n"),
@@ -1170,7 +1171,7 @@ alSatisfiesDepend(const availableList al,
 	return ret;
     }
     return NULL;
- }
+}
 
 /**
  * Check key for an unsatisfied dependency.
@@ -1610,13 +1611,12 @@ static int checkDependentConflicts(rpmTransactionSet ts,
     return rc;
 }
 
-/*
- * XXX Hack to remove known Red Hat dependency loops, will be removed
- * as soon as rpm's legacy permits.
- */
-#define	DEPENDENCY_WHITEOUT
+struct badDeps_s {
+/*@observer@*/ /*@null@*/ const char * pname;
+/*@observer@*/ /*@null@*/ const char * qname;
+};
 
-#if defined(DEPENDENCY_WHITEOUT)
+#ifdef	DYING
 static struct badDeps_s {
 /*@observer@*/ /*@null@*/ const char * pname;
 /*@observer@*/ /*@null@*/ const char * qname;
@@ -1645,28 +1645,70 @@ static struct badDeps_s {
     { "pam", "pamconfig" },
     { NULL, NULL }
 };
-    
+#else
+static struct badDeps_s * badDeps = NULL;
+#endif
+
+/**
+ * Check for dependency relations to be ignored.
+ *
+ * @param p	successor package (i.e. with Requires: )
+ * @param q	predecessor package (i.e. with Provides: )
+ * @return	1 if dependency is to be ignored.
+ */
 static int ignoreDep(const struct availablePackage * p,
 		const struct availablePackage * q)
 	/*@*/
 {
-    struct badDeps_s * bdp = badDeps;
+    struct badDeps_s * bdp;
+    static int _initialized = 0;
+    const char ** av = NULL;
+    int ac = 0;
 
-    while (bdp->pname != NULL && bdp->qname != NULL) {
+    if (!_initialized) {
+	char * s = rpmExpand("%{?_dependency_whiteout}", NULL);
+	int i;
+
+	if (s != NULL && *s != '\0'
+	&& !(i = poptParseArgvString(s, &ac, (const char ***)&av))
+	&& ac > 0 && av != NULL)
+	{
+	    bdp = badDeps = xcalloc(ac+1, sizeof(*badDeps));
+	    for (i = 0; i < ac; i++, bdp++) {
+		char * p, * q;
+
+		if (av[i] == NULL)
+		    break;
+		p = xstrdup(av[i]);
+		if ((q = strchr(p, '>')) != NULL)
+		    *q++ = '\0';
+		bdp->pname = p;
+		bdp->qname = q;
+		rpmMessage(RPMMESS_DEBUG,
+			_("ignore package name relation(s) [%d]\t%s -> %s\n"),
+			i, bdp->pname, bdp->qname);
+	    }
+	    bdp->pname = bdp->qname = NULL;
+	}
+	av = _free(av);
+	s = _free(s);
+	_initialized++;
+    }
+
+    if (badDeps != NULL)
+    for (bdp = badDeps; bdp->pname != NULL && bdp->qname != NULL; bdp++) {
 	if (!strcmp(p->name, bdp->pname) && !strcmp(q->name, bdp->qname))
 	    return 1;
-	bdp++;
     }
     return 0;
 }
-#endif
 
 /**
  * Recursively mark all nodes with their predecessors.
  * @param tsi		successor chain
  * @param q		predecessor
  */
-static void markLoop(/*@special@*/ struct tsortInfo * tsi,
+static void markLoop(/*@special@*/ tsortInfo tsi,
 		struct availablePackage * q)
 	/*@uses tsi @*/
 	/*@modifies internalState @*/
@@ -1719,8 +1761,8 @@ zapRelation(struct availablePackage * q, struct availablePackage * p,
 		int zap, /*@in@*/ /*@out@*/ int * nzaps)
 	/*@modifies q, p, *nzaps @*/
 {
-    struct tsortInfo * tsi_prev;
-    struct tsortInfo * tsi;
+    tsortInfo tsi_prev;
+    tsortInfo tsi;
     const char *dp = NULL;
 
     for (tsi_prev = &q->tsi, tsi = q->tsi.tsi_next;
@@ -1778,7 +1820,7 @@ static inline int addRelation( const rpmTransactionSet ts,
 	/*@modifies p->tsi.tsi_u.count, p->depth, *selected @*/
 {
     struct availablePackage * q;
-    struct tsortInfo * tsi;
+    tsortInfo tsi;
     int matchNum;
 
     if (!p->requires || !p->requiresEVR || !p->requireFlags)
@@ -1795,11 +1837,9 @@ static inline int addRelation( const rpmTransactionSet ts,
     if (!strncmp(p->requires[j], "rpmlib(", sizeof("rpmlib(")-1))
 	return 0;
 
-#if defined(DEPENDENCY_WHITEOUT)
     /* Avoid certain dependency relations. */
     if (ignoreDep(p, q))
 	return 0;
-#endif
 
     /* Avoid redundant relations. */
     /* XXX TODO: add control bit. */
@@ -1848,6 +1888,9 @@ static void addQ(struct availablePackage * p,
 {
     struct availablePackage *q, *qprev;
 
+    /* Mark the package as queued. */
+    p->tsi.tsi_reqx = 1;
+
     if ((*rp) == NULL) {	/* 1st element */
 	(*rp) = (*qp) = p;
 	return;
@@ -1871,21 +1914,27 @@ static void addQ(struct availablePackage * p,
 int rpmdepOrder(rpmTransactionSet ts)
 {
     int npkgs = ts->addedPackages.size;
+#ifdef	DYING
     int chainsaw = ts->transFlags & RPMTRANS_FLAG_CHAINSAW;
+#else
+    int chainsaw = 1;
+#endif
     struct availablePackage * p;
     struct availablePackage * q;
     struct availablePackage * r;
-    struct tsortInfo * tsi;
-    struct tsortInfo * tsi_next;
+    tsortInfo tsi;
+    tsortInfo tsi_next;
     int * ordering = alloca(sizeof(*ordering) * (npkgs + 1));
     int orderingCount = 0;
     unsigned char * selected = alloca(sizeof(*selected) * (npkgs + 1));
     int loopcheck;
-    struct transactionElement * newOrder;
+    transactionElement newOrder;
     int newOrderCount = 0;
     struct orderListIndex * orderList;
     int nrescans = 10;
     int _printed = 0;
+    int treex;
+    int depth;
     int qlen;
     int i, j;
 
@@ -1947,14 +1996,16 @@ int rpmdepOrder(rpmTransactionSet ts)
 	}
     }
 
-    /* Save predecessor count. */
+    /* Save predecessor count and mark tree roots. */
+    treex = 0;
     if ((p = ts->addedPackages.list) != NULL)
     for (i = 0; i < npkgs; i++, p++) {
 	p->npreds = p->tsi.tsi_count;
+	p->tree = (p->npreds == 0 ? treex++ : -1);
     }
 
     /* T4. Scan for zeroes. */
-    rpmMessage(RPMMESS_DEBUG, _("========== tsorting packages (order, #predecessors, #succesors, depth)\n"));
+    rpmMessage(RPMMESS_DEBUG, _("========== tsorting packages (order, #predecessors, #succesors, tree, depth)\n"));
 
 rescan:
     q = r = NULL;
@@ -1962,7 +2013,7 @@ rescan:
     if ((p = ts->addedPackages.list) != NULL)
     for (i = 0; i < npkgs; i++, p++) {
 
-	/* Prefer packages in presentation order. */
+	/* Prefer packages in chainsaw or presentation order. */
 	if (!chainsaw)
 	    p->tsi.tsi_qcnt = (npkgs - i);
 
@@ -1976,10 +2027,19 @@ rescan:
     /* T5. Output front of queue (T7. Remove from queue.) */
     for (; q != NULL; q = q->tsi.tsi_suc) {
 
-	rpmMessage(RPMMESS_DEBUG, "%5d%5d%5d%3d %*s %s-%s-%s\n",
-			orderingCount, q->npreds, q->tsi.tsi_qcnt, q->depth,
+	/* Mark the package as unqueued. */
+	q->tsi.tsi_reqx = 0;
+
+	rpmMessage(RPMMESS_DEBUG, "%5d%5d%5d%5d%5d %*s %s-%s-%s\n",
+			orderingCount, q->npreds, q->tsi.tsi_qcnt,
+			q->tree, q->depth,
 			2*q->depth, "",
 			q->name, q->version, q->release);
+
+	treex = q->tree;
+	depth = q->depth;
+	q->degree = 0;
+
 	ordering[orderingCount++] = q - ts->addedPackages.list;
 	qlen--;
 	loopcheck--;
@@ -1992,6 +2052,12 @@ rescan:
 	    tsi->tsi_next = NULL;
 	    p = tsi->tsi_suc;
 	    if (p && (--p->tsi.tsi_count) <= 0) {
+
+		p->tree = treex;
+		p->depth = depth + 1;
+		p->parent = q;
+		q->degree++;
+
 		/* XXX TODO: add control bit. */
 		p->tsi.tsi_suc = NULL;
 		/*@-nullstate@*/	/* FIX: q->tsi.tsi_u.suc may be NULL */
@@ -2005,6 +2071,18 @@ rescan:
 	    _printed++;
 	    rpmMessage(RPMMESS_DEBUG,
 		_("========== successors only (presentation order)\n"));
+
+	    /* Relink the queue in presentation order. */
+	    tsi = &q->tsi;
+	    if ((p = ts->addedPackages.list) != NULL)
+	    for (i = 0; i < npkgs; i++, p++) {
+		/* Is this element in the queue? */
+		if (p->tsi.tsi_reqx == 0)
+		    /*@innercontinue@*/ continue;
+		tsi->tsi_suc = p;
+		tsi = &p->tsi;
+	    }
+	    tsi->tsi_suc = NULL;
 	}
     }
 
