@@ -419,7 +419,7 @@ static int saveHardLink(/*@special@*/ /*@partial@*/ FSM_t fsm)
  * Destroy set of hard links.
  * @param li		set of hard links
  */
-static /*@null@*/ void * freeHardLink(/*@only@*/ /*@null@*/ struct hardLink * li)
+static /*@null@*/ void * freeHardLink(/*@only@*/ /*@null@*/ struct hardLink_s * li)
 	/*@modifies li @*/
 {
     if (li) {
@@ -648,7 +648,14 @@ int fsmMapAttrs(FSM_t fsm)
 	if (fsm->mapFlags & CPIO_MAP_GID)
 	    st->st_gid = finalGid;
 
-	fsm->fmd5sum = (fi->fmd5s ? fi->fmd5s[i] : NULL);
+	{   rpmTransactionSet ts = fsmGetTs(fsm);
+
+	    if (ts != NULL && !(ts->transFlags & RPMTRANS_FLAG_NOMD5)) {
+		fsm->fmd5sum = (fi->fmd5s ? fi->fmd5s[i] : NULL);
+	    } else {
+		fsm->fmd5sum = NULL;
+	    }
+	}
 
     }
     return 0;
@@ -678,7 +685,7 @@ static int expandRegular(/*@special@*/ FSM_t fsm)
 
     /* XXX This doesn't support brokenEndian checks. */
     if (st->st_size > 0 && fmd5sum)
-	fdInitMD5(fsm->wfd, 0);
+	fdInitDigest(fsm->wfd, PGPHASHALGO_MD5, 0);
 
     while (left) {
 
@@ -702,7 +709,7 @@ static int expandRegular(/*@special@*/ FSM_t fsm)
 	const char * md5sum = NULL;
 
 	(void) Fflush(fsm->wfd);
-	fdFiniMD5(fsm->wfd, (void **)&md5sum, NULL, 1);
+	fdFiniDigest(fsm->wfd, PGPHASHALGO_MD5, (void **)&md5sum, NULL, 1);
 
 	if (md5sum == NULL) {
 	    rc = CPIOERR_MD5SUM_MISMATCH;
@@ -735,6 +742,7 @@ static int writeFile(/*@special@*/ FSM_t fsm, int writeData)
     size_t pos = fdGetCpioPos(fsm->cfd);
     char * symbuf = NULL;
     int left;
+    int xx;
     int rc;
 
     st->st_size = (writeData ? ost->st_size : 0);
@@ -789,6 +797,9 @@ static int writeFile(/*@special@*/ FSM_t fsm, int writeData)
 	    rdbuf = fsm->rdbuf;
 	    fsm->rdbuf = (char *) mapped;
 	    fsm->rdlen = nmapped = st->st_size;
+#if defined(MADV_DONTNEED)
+	    xx = madvise(mapped, nmapped, MADV_DONTNEED);
+#endif
 	}
 #endif
 
@@ -815,6 +826,9 @@ static int writeFile(/*@special@*/ FSM_t fsm, int writeData)
 
 #if HAVE_MMAP
 	if (mapped != (void *)-1) {
+#if defined(MADV_DONTNEED)
+	    xx = madvise(mapped, nmapped, MADV_DONTNEED);
+#endif
 	    /*@-noeffect@*/ (void) munmap(mapped, nmapped) /*@=noeffect@*/;
 	    fsm->rdbuf = rdbuf;
 	}
@@ -926,11 +940,13 @@ static int fsmMakeLinks(/*@special@*/ FSM_t fsm)
     fsm->path = NULL;
     for (i = 0; i < fsm->li->nlink; i++) {
 	if (fsm->li->filex[i] < 0) continue;
-	if (i == fsm->li->createdPath) continue;
+	if (fsm->li->createdPath == i) continue;
 
 	fsm->ix = fsm->li->filex[i];
 	fsm->path = _free(fsm->path);
 	rc = fsmStage(fsm, FSM_MAP);
+	if (XFA_SKIPPING(fsm->action)) continue;
+
 	rc = fsmStage(fsm, FSM_VERIFY);
 	if (!rc) continue;
 	if (rc != CPIOERR_LSTAT_FAILED) break;
@@ -984,7 +1000,8 @@ static int fsmCommitLinks(/*@special@*/ FSM_t fsm)
 	if (fsm->li->filex[i] < 0) continue;
 	fsm->ix = fsm->li->filex[i];
 	rc = fsmStage(fsm, FSM_MAP);
-	rc = fsmStage(fsm, FSM_COMMIT);
+	if (!XFA_SKIPPING(fsm->action))
+	    rc = fsmStage(fsm, FSM_COMMIT);
 	fsm->path = _free(fsm->path);
 	fsm->li->filex[i] = -1;
     }
@@ -1485,7 +1502,7 @@ int fsmStage(FSM_t fsm, fileStage stage)
 
 	if (fsm->goal == FSM_PKGBUILD) {
 	    if (!S_ISDIR(st->st_mode) && st->st_nlink > 1) {
-		struct hardLink * li, * prev;
+		struct hardLink_s * li, * prev;
 
 if (!(fsm->mapFlags & CPIO_ALL_HARDLINKS)) break;
 		rc = writeLinkedFile(fsm);
@@ -1582,7 +1599,9 @@ if (!(fsm->mapFlags & CPIO_ALL_HARDLINKS)) break;
 	    if (rc == CPIOERR_LSTAT_FAILED)
 		rc = fsmStage(fsm, FSM_MKNOD);
 	} else {
-	    rc = CPIOERR_UNKNOWN_FILETYPE;
+	    /* XXX Special case /dev/log, which shouldn't be packaged anyways */
+	    if (strncmp(fsm->path, "/dev/log", sizeof("/dev/log")-1))
+		rc = CPIOERR_UNKNOWN_FILETYPE;
 	}
 	if (!S_ISDIR(st->st_mode) && st->st_nlink > 1) {
 	    fsm->li->createdPath = fsm->li->linkIndex;
@@ -1699,7 +1718,10 @@ if (!(fsm->mapFlags & CPIO_ALL_HARDLINKS)) break;
 	    break;
 	}
 
-	if (!S_ISSOCK(st->st_mode)) {	/* XXX /dev/log et al are skipped */
+	/* XXX Special case /dev/log, which shouldn't be packaged anyways */
+	if (!S_ISSOCK(st->st_mode)
+	&&  strncmp(fsm->path, "/dev/log", sizeof("/dev/log")-1))
+	{
 	    /* Rename temporary to final file name. */
 	    if (!S_ISDIR(st->st_mode) &&
 		(fsm->subdir || fsm->suffix || fsm->nsuffix))
@@ -2122,11 +2144,11 @@ if (!(fsm->mapFlags & CPIO_ALL_HARDLINKS)) break;
     switch(a) {
     case FSM_UNKNOWN:	return "unknown";
 
-    case FSM_PKGINSTALL:return "pkginstall";
-    case FSM_PKGERASE:	return "pkgerase";
-    case FSM_PKGBUILD:	return "pkgbuild";
-    case FSM_PKGCOMMIT:	return "pkgcommit";
-    case FSM_PKGUNDO:	return "pkgundo";
+    case FSM_PKGINSTALL:return "INSTALL";
+    case FSM_PKGERASE:	return "ERASE";
+    case FSM_PKGBUILD:	return "BUILD";
+    case FSM_PKGCOMMIT:	return "COMMIT";
+    case FSM_PKGUNDO:	return "UNDO";
 
     case FSM_CREATE:	return "create";
     case FSM_INIT:	return "init";
