@@ -6,19 +6,25 @@
  * File state machine to handle a payload within an rpm package.
  */
 
-#include <rpmlib.h>
 #include "cpio.h"
 
+/*@-exportlocal@*/
+/*@unchecked@*/
+extern int _fsm_debug;
+/*@=exportlocal@*/
+
 /**
  */
 #define	FSM_VERBOSE	0x8000
 #define	FSM_INTERNAL	0x4000
 #define	FSM_SYSCALL	0x2000
 #define	FSM_DEAD	0x1000
+
 #define	_fv(_a)		((_a) | FSM_VERBOSE)
 #define	_fi(_a)		((_a) | FSM_INTERNAL)
 #define	_fs(_a)		((_a) | (FSM_INTERNAL | FSM_SYSCALL))
 #define	_fd(_a)		((_a) | (FSM_INTERNAL | FSM_DEAD))
+
 typedef enum fileStage_e {
     FSM_UNKNOWN =   0,
     FSM_INIT	=  _fd(1),
@@ -76,7 +82,7 @@ typedef enum fileStage_e {
     FSM_RCLOSE	=  _fs(131),
     FSM_WOPEN	=  _fs(132),
     FSM_WRITE	=  _fs(133),
-    FSM_WCLOSE	=  _fs(134),
+    FSM_WCLOSE	=  _fs(134)
 } fileStage;
 #undef	_fv
 #undef	_fi
@@ -86,8 +92,8 @@ typedef enum fileStage_e {
 /** \ingroup payload
  * Keeps track of the set of all hard links to a file in an archive.
  */
-struct hardLink {
-/*@owned@*/ struct hardLink * next;
+struct hardLink_s {
+/*@owned@*/ struct hardLink_s * next;
 /*@owned@*/ const char ** nsuffix;
 /*@owned@*/ int * filex;
     struct stat sb;
@@ -129,8 +135,8 @@ struct fsm_s {
     size_t wrnb;			/*!< write: Number of bytes returned. */
 /*@only@*/ FSMI_t iter;			/*!< File iterator. */
     int ix;				/*!< Current file iterator index. */
-/*@only@*/ struct hardLink * links;	/*!< Pending hard linked file(s). */
-/*@only@*/ struct hardLink * li;	/*!< Current hard linked file(s). */
+/*@only@*/ struct hardLink_s * links;	/*!< Pending hard linked file(s). */
+/*@only@*/ struct hardLink_s * li;	/*!< Current hard linked file(s). */
 /*@kept@*/ /*@null@*/ unsigned int * archiveSize;	/*!< Pointer to archive size. */
 /*@kept@*/ /*@null@*/ const char ** failedFile;	/*!< First file name that failed. */
 /*@shared@*/ const char * subdir;	/*!< Current file sub-directory. */
