@@ -13,13 +13,15 @@
 extern time_t get_date(const char * p, void * now);	/* XXX expedient lies */
 /*@=redecl@*/
 
+/*@unchecked@*/
 struct rpmInstallArguments_s rpmIArgs;
 
-#define	POPT_RELOCATE		1016
-#define	POPT_EXCLUDEPATH	1019
-#define	POPT_ROLLBACK		1024
+#define	POPT_RELOCATE		-1016
+#define	POPT_EXCLUDEPATH	-1019
+#define	POPT_ROLLBACK		-1024
 
 /*@exits@*/ static void argerror(const char * desc)
+	/*@globals fileSystem @*/
 	/*@modifies fileSystem @*/
 {
     fprintf(stderr, _("%s: %s\n"), __progname, desc);
@@ -37,6 +39,12 @@ static void installArgCallback( /*@unused@*/ poptContext con,
 {
     struct rpmInstallArguments_s * ia = &rpmIArgs;
 
+#if 0
+fprintf(stderr, "*** opt %s %c info 0x%x arg %p val 0x%x arg %p %s\n", opt->longName, opt->shortName, opt->argInfo, opt->arg, opt->val, arg, arg);
+#endif
+
+    /* XXX avoid accidental collisions with POPT_BIT_SET for flags */
+    if (opt->arg == NULL)
     switch (opt->val) {
     case POPT_EXCLUDEPATH:
 	if (arg == NULL || *arg != '/') 
@@ -70,11 +78,16 @@ static void installArgCallback( /*@unused@*/ poptContext con,
       }	break;
     case POPT_ROLLBACK:
       {	time_t tid;
+	if (arg == NULL) 
+	    argerror(_("rollback takes a time/date stamp argument"));
 	tid = get_date(arg, NULL);
-	if (tid == (time_t)-1)
-	    argerror(_("malformed rollback time"));
+
+	if (tid == (time_t)-1 || tid == (time_t)0)
+	    argerror(_("malformed rollback time/date stamp argument"));
 	ia->rbtid = tid;
       }	break;
+    default:
+	break;
     }
 }
 
@@ -101,6 +114,9 @@ struct poptOption rpmInstallPoptTable[] = {
  { "badreloc", '\0', POPT_BIT_SET,
 	&rpmIArgs.probFilter, RPMPROB_FILTER_FORCERELOCATE,
 	N_("relocate files in non-relocateable package"), NULL},
+ { "chainsaw", '\0', POPT_BIT_SET|POPT_ARGFLAG_DOC_HIDDEN,
+	&rpmIArgs.transFlags, RPMTRANS_FLAG_CHAINSAW,
+	N_("use chainsaw dependency tree decimation when ordering"), NULL},
  { "dirstash", '\0', POPT_BIT_SET|POPT_ARGFLAG_DOC_HIDDEN,
 	&rpmIArgs.transFlags, RPMTRANS_FLAG_DIRSTASH,
 	N_("save erased package files by renaming into sub-directory"), NULL},
@@ -200,7 +216,7 @@ struct poptOption rpmInstallPoptTable[] = {
 	&rpmIArgs.probFilter, RPMPROB_FILTER_REPLACEPKG,
 	N_("reinstall if the package is already present"), NULL},
  { "rollback", '\0', POPT_ARG_STRING|POPT_ARGFLAG_DOC_HIDDEN, 0, POPT_ROLLBACK,
-	N_("deinstall new package(s), reinstall old package(s), back to date"),
+	N_("deinstall new, reinstall old, package(s), back to <date>"),
 	N_("<date>") },
  { "test", '\0', POPT_BIT_SET, &rpmIArgs.transFlags, RPMTRANS_FLAG_TEST,
 	N_("don't install, but tell if it would work or not"), NULL},
