@@ -10,16 +10,23 @@
 
 #include "debug.h"
 
+/*@unchecked@*/
 struct rpmQVArguments_s rpmQVArgs;
+
+/*@unchecked@*/
 int specedit = 0;
 
-#define POPT_QUERYFORMAT	1000
-#define POPT_WHATREQUIRES	1001
-#define POPT_WHATPROVIDES	1002
-#define POPT_QUERYBYNUMBER	1003
-#define POPT_TRIGGEREDBY	1004
-#define POPT_DUMP		1005
-#define POPT_SPECFILE		1006
+#define POPT_QUERYFORMAT	-1000
+#define POPT_WHATREQUIRES	-1001
+#define POPT_WHATPROVIDES	-1002
+#define POPT_QUERYBYNUMBER	-1003
+#define POPT_TRIGGEREDBY	-1004
+#define POPT_DUMP		-1005
+#define POPT_SPECFILE		-1006
+#define POPT_QUERYBYPKGID	-1007
+#define POPT_QUERYBYHDRID	-1008
+#define POPT_QUERYBYFILEID	-1009
+#define POPT_QUERYBYTID		-1010
 
 /* ========== Query/Verify source popt args */
 static void rpmQVSourceArgCallback( /*@unused@*/ poptContext con,
@@ -49,6 +56,14 @@ static void rpmQVSourceArgCallback( /*@unused@*/ poptContext con,
 			      qva->qva_sourceCount++; break;
     case POPT_TRIGGEREDBY: qva->qva_source |= RPMQV_TRIGGEREDBY;
 			      qva->qva_sourceCount++; break;
+    case POPT_QUERYBYPKGID: qva->qva_source |= RPMQV_PKGID;
+			      qva->qva_sourceCount++; break;
+    case POPT_QUERYBYHDRID: qva->qva_source |= RPMQV_HDRID;
+			      qva->qva_sourceCount++; break;
+    case POPT_QUERYBYFILEID: qva->qva_source |= RPMQV_FILEID;
+			      qva->qva_sourceCount++; break;
+    case POPT_QUERYBYTID: qva->qva_source |= RPMQV_TID;
+			      qva->qva_sourceCount++; break;
 
 /* XXX SPECFILE is not verify sources */
     case POPT_SPECFILE:
@@ -65,6 +80,7 @@ static void rpmQVSourceArgCallback( /*@unused@*/ poptContext con,
 /**
  * Common query/verify mode options.
  */
+/*@unchecked@*/
 struct poptOption rpmQVSourcePoptTable[] = {
  { NULL, '\0', POPT_ARG_CALLBACK | POPT_CBFLAG_INC_DATA, 
 	rpmQVSourceArgCallback, 0, NULL, NULL },
@@ -72,10 +88,16 @@ struct poptOption rpmQVSourcePoptTable[] = {
 	N_("query/verify all packages"), NULL },
  { "file", 'f', 0, 0, 'f',
 	N_("query/verify package(s) owning file"), "FILE" },
+ { "fileid", '\0', POPT_ARGFLAG_DOC_HIDDEN, 0, POPT_QUERYBYFILEID, 
+	N_("query/verify package(s) with file identifier"), "MD5" },
  { "group", 'g', 0, 0, 'g',
 	N_("query/verify package(s) in group"), "GROUP" },
+ { "hdrid", '\0', POPT_ARGFLAG_DOC_HIDDEN, 0, POPT_QUERYBYHDRID, 
+	N_("query/verify package(s) with header identifier"), "SHA1" },
  { "package", 'p', 0, 0, 'p',
 	N_("query/verify a package file (i.e. a binary *.rpm file)"), NULL },
+ { "pkgid", '\0', POPT_ARGFLAG_DOC_HIDDEN, 0, POPT_QUERYBYPKGID, 
+	N_("query/verify package(s) with package identifier"), "MD5" },
  { "query", 'q', POPT_ARGFLAG_DOC_HIDDEN, NULL, 'q',
 	N_("rpm query mode"), NULL },
  { "querybynumber", '\0', POPT_ARGFLAG_DOC_HIDDEN, 0, 
@@ -84,6 +106,8 @@ struct poptOption rpmQVSourcePoptTable[] = {
 	N_("display known query tags"), NULL },
  { "specfile", '\0', 0, 0, POPT_SPECFILE,
 	N_("query a spec file"), N_("<spec>") },
+ { "tid", '\0', POPT_ARGFLAG_DOC_HIDDEN, 0, POPT_QUERYBYTID, 
+	N_("query/verify package(s) from install transaction"), "TID" },
  { "triggeredby", '\0', POPT_ARGFLAG_DOC_HIDDEN, 0, POPT_TRIGGEREDBY, 
 	N_("query the package(s) triggered by the package"), "PACKAGE" },
  { "verify", 'V', POPT_ARGFLAG_DOC_HIDDEN, NULL, 'V',
@@ -137,6 +161,7 @@ static void queryArgCallback(/*@unused@*/poptContext con,
 /**
  * Query mode options.
  */
+/*@unchecked@*/
 struct poptOption rpmQueryPoptTable[] = {
  { NULL, '\0', POPT_ARG_CALLBACK | POPT_CBFLAG_INC_DATA, 
 	queryArgCallback, 0, NULL, NULL },
@@ -180,6 +205,7 @@ struct poptOption rpmQueryPoptTable[] = {
 /**
  * Verify mode options.
  */
+/*@unchecked@*/
 struct poptOption rpmVerifyPoptTable[] = {
 #ifdef	DYING
  { NULL, '\0', POPT_ARG_CALLBACK | POPT_CBFLAG_INC_DATA, 
