@@ -29,6 +29,7 @@ static void printFileInfo(char * te, const char * name,
 			  unsigned short rdev, unsigned int nlink,
 			  const char * owner, const char * group,
 			  int uid, int gid, const char * linkto)
+	/*@modifies *te @*/
 {
     char sizefield[15];
     char ownerfield[9], groupfield[9];
@@ -108,6 +109,7 @@ static void printFileInfo(char * te, const char * name,
 /**
  */
 static inline /*@null@*/ const char * queryHeader(Header h, const char * qfmt)
+	/*@*/
 {
     const char * errstr;
     const char * str;
@@ -122,6 +124,7 @@ static inline /*@null@*/ const char * queryHeader(Header h, const char * qfmt)
  */
 static int countLinks(int_16 * fileRdevList, int_32 * fileInodeList, int nfiles,
 		int xfile)
+	/*@*/
 {
     int nlink = 0;
 
@@ -189,6 +192,7 @@ int showQueryPackage(QVA_t qva, /*@unused@*/rpmdb rpmdb, Header h)
     if (queryFormat) {
 	const char * str = queryHeader(h, queryFormat);
 	nonewline = 1;
+	/*@-branchstate@*/
 	if (str) {
 	    size_t tb = (te - t);
 	    size_t sb = strlen(str);
@@ -202,6 +206,7 @@ int showQueryPackage(QVA_t qva, /*@unused@*/rpmdb rpmdb, Header h)
 	    /*@=usereleased@*/
 	    str = _free(str);
 	}
+	/*@=branchstate@*/
     }
 
     if (!(queryFlags & QUERY_FOR_LIST))
@@ -259,25 +264,31 @@ int showQueryPackage(QVA_t qva, /*@unused@*/rpmdb rpmdb, Header h)
 	  && (fileFlagsList[i] & RPMFILE_GHOST))
 	    continue;
 
+	/*@-internalglobs@*/ /* FIX: shrug */
 	if (!rpmIsVerbose() && prefix)
 	    te = stpcpy(te, prefix);
+	/*@=internalglobs@*/
 
 	if (queryFlags & QUERY_FOR_STATE) {
 	    if (fileStatesList) {
 		rpmfileState fstate = fileStatesList[i];
 		switch (fstate) {
 		case RPMFILE_STATE_NORMAL:
-		    te = stpcpy(te, _("normal        ")); break;
+		    te = stpcpy(te, _("normal        "));
+		    /*@switchbreak@*/ break;
 		case RPMFILE_STATE_REPLACED:
-		    te = stpcpy(te, _("replaced      ")); break;
+		    te = stpcpy(te, _("replaced      "));
+		    /*@switchbreak@*/ break;
 		case RPMFILE_STATE_NOTINSTALLED:
-		    te = stpcpy(te, _("not installed ")); break;
+		    te = stpcpy(te, _("not installed "));
+		    /*@switchbreak@*/ break;
 		case RPMFILE_STATE_NETSHARED:
-		    te = stpcpy(te, _("net shared    ")); break;
+		    te = stpcpy(te, _("net shared    "));
+		    /*@switchbreak@*/ break;
 		default:
 		    sprintf(te, _("(unknown %3d) "), (int)fileStatesList[i]);
 		    te += strlen(te);
-		    break;
+		    /*@switchbreak@*/ break;
 		}
 	    } else {
 		te = stpcpy(te, _("(no state)    "));
@@ -313,10 +324,14 @@ int showQueryPackage(QVA_t qva, /*@unused@*/rpmdb rpmdb, Header h)
 	    else
 		sprintf(te, "X");
 	    te += strlen(te);
-	} else if (!rpmIsVerbose()) {
+	} else
+	/*@-internalglobs@*/ /* FIX: shrug */
+	if (!rpmIsVerbose()) {
 	    te = stpcpy(te, dirNames[dirIndexes[i]]);
 	    te = stpcpy(te, baseNames[i]);
-	} else {
+	}
+	/*@=internalglobs@*/
+	else {
 	    char * filespec;
 	    int nlink;
 	    size_t fileSize;
@@ -389,6 +404,8 @@ exit:
  */
 static void
 printNewSpecfile(Spec spec)
+	/*@globals fileSystem @*/
+	/*@modifies spec->sl->sl_lines[], fileSystem @*/
 {
     Header h;
     speclines sl = spec->sl;
@@ -399,6 +416,7 @@ printNewSpecfile(Spec spec)
     if (sl == NULL || st == NULL)
 	return;
 
+    /*@-branchstate@*/
     for (i = 0; i < st->st_ntags; i++) {
 	spectag t = st->st_t + i;
 	const char * tn = tagName(t->t_tag);
@@ -453,11 +471,11 @@ printNewSpecfile(Spec spec)
 		(void) stpcpy( stpcpy( stpcpy(buf, tn), ": "), msgstr);
 		sl->sl_lines[t->t_startx] = buf;
 	    }
-	    break;
+	    /*@switchbreak@*/ break;
 	case RPMTAG_DESCRIPTION:
 	    for (j = 1; j < t->t_nlines; j++) {
 		if (*sl->sl_lines[t->t_startx + j] == '%')
-		    continue;
+		    /*@innercontinue@*/ continue;
 		/*@-unqualifiedtrans@*/
 		sl->sl_lines[t->t_startx + j] =
 			_free(sl->sl_lines[t->t_startx + j]);
@@ -470,9 +488,10 @@ printNewSpecfile(Spec spec)
 	    sl->sl_lines[t->t_startx + 1] = xstrdup(msgstr);
 	    if (t->t_nlines > 2)
 		sl->sl_lines[t->t_startx + 2] = xstrdup("\n\n");
-	    break;
+	    /*@switchbreak@*/ break;
 	}
     }
+    /*@=branchstate@*/
     msgstr = _free(msgstr);
 
     for (i = 0; i < sl->sl_nlines; i++) {
@@ -502,7 +521,7 @@ void rpmDisplayQueryTags(FILE * fp)
 	/* XXX don't print query tags twice. */
 	for (i = 0, t = rpmTagTable; i < rpmTagTableSize; i++, t++) {
 	    if (t->name == NULL)	/* XXX programmer error. */
-		continue;
+		/*@innercontinue@*/ continue;
 	    if (!strcmp(t->name, ext->name))
 	    	/*@innerbreak@*/ break;
 	}
@@ -528,6 +547,23 @@ int showMatches(QVA_t qva, rpmdbMatchIterator mi, QVF_t showPackage)
     return ec;
 }
 
+/**
+ * Convert hex to binary nibble.
+ * @param c		hex character
+ * @return		binary nibble
+ */
+static inline unsigned char nibble(char c)
+	/*@*/
+{
+    if (c >= '0' && c <= '9')
+	return (c - '0');
+    if (c >= 'A' && c <= 'F')
+	return (c - 'A') + 10;
+    if (c >= 'a' && c <= 'f')
+	return (c - 'a') + 10;
+    return 0;
+}
+
 /*@-redecl@*/
 /**
  * @todo Eliminate linkage loop into librpmbuild.a
@@ -551,13 +587,14 @@ int rpmQueryVerify(QVA_t qva, rpmQVSources source, const char * arg,
     int retcode = 0;
     const char ** av = NULL;
     char * end = NULL;
+    const char * s;
+    int i;
 
     switch (source) {
     case RPMQV_RPM:
     {	int ac = 0;
 	const char * fileURL = NULL;
 	rpmRC rpmrc;
-	int i;
 
 	rc = rpmGlob(arg, &ac, &av);
 	if (rc) return 1;
@@ -717,6 +754,95 @@ restart:
 	}
 	break;
 
+    case RPMQV_PKGID:
+    {	unsigned char md5[16];
+	unsigned char * t;
+
+	for (i = 0, s = arg; *s && isxdigit(*s); s++, i++)
+	    {};
+	if (i != 32) {
+	    rpmError(RPMERR_QUERYINFO, _("malformed %s: %s\n"), "pkgid", arg);
+	    return 1;
+	}
+
+        for (i = 0, t = md5, s = arg; i < 16; i++, t++, s += 2)
+            *t = (nibble(s[0]) << 4) | nibble(s[1]);
+	
+	mi = rpmdbInitIterator(rpmdb, RPMTAG_SIGMD5, md5, 16);
+	if (mi == NULL) {
+	    rpmError(RPMERR_QUERYINFO, _("no package matches %s: %s\n"),
+			"pkgid", arg);
+	    retcode = 1;
+	} else {
+	    retcode = showMatches(qva, mi, showPackage);
+	}
+    }	break;
+
+    case RPMQV_HDRID:
+	for (i = 0, s = arg; *s && isxdigit(*s); s++, i++)
+	    {};
+	if (i != 40) {
+	    rpmError(RPMERR_QUERYINFO, _("malformed %s: %s\n"), "hdrid", arg);
+	    return 1;
+	}
+
+	mi = rpmdbInitIterator(rpmdb, RPMTAG_SHA1HEADER, arg, 0);
+	if (mi == NULL) {
+	    rpmError(RPMERR_QUERYINFO, _("no package matches %s: %s\n"),
+			"hdrid", arg);
+	    retcode = 1;
+	} else {
+	    retcode = showMatches(qva, mi, showPackage);
+	}
+	break;
+
+    case RPMQV_FILEID:
+	for (i = 0, s = arg; *s && isxdigit(*s); s++, i++)
+	    {};
+	if (i != 32) {
+	    rpmError(RPMERR_QUERY, _("malformed %s: %s\n"), "fileid", arg);
+	    return 1;
+	}
+
+	mi = rpmdbInitIterator(rpmdb, RPMTAG_FILEMD5S, arg, 0);
+	if (mi == NULL) {
+	    rpmError(RPMERR_QUERYINFO, _("no package matches %s: %s\n"),
+			"fileid", arg);
+	    retcode = 1;
+	} else {
+	    retcode = showMatches(qva, mi, showPackage);
+	}
+	break;
+
+    case RPMQV_TID:
+    {	int mybase = 10;
+	const char * myarg = arg;
+	unsigned iid;
+
+	/* XXX should be in strtoul */
+	if (*myarg == '0') {
+	    myarg++;
+	    mybase = 8;
+	    if (*myarg == 'x') {
+		myarg++;
+		mybase = 16;
+	    }
+	}
+	iid = strtoul(myarg, &end, mybase);
+	if ((*end) || (end == arg) || (iid == ULONG_MAX)) {
+	    rpmError(RPMERR_QUERY, _("malformed %s: %s\n"), "tid", arg);
+	    return 1;
+	}
+	mi = rpmdbInitIterator(rpmdb, RPMTAG_INSTALLTID, &iid, sizeof(iid));
+	if (mi == NULL) {
+	    rpmError(RPMERR_QUERYINFO, _("no package matches %s: %s\n"),
+			"tid", arg);
+	    retcode = 1;
+	} else {
+	    retcode = showMatches(qva, mi, showPackage);
+	}
+    }	break;
+
     case RPMQV_WHATREQUIRES:
 	mi = rpmdbInitIterator(rpmdb, RPMTAG_REQUIRENAME, arg, 0);
 	if (mi == NULL) {
@@ -740,8 +866,7 @@ restart:
 	}
 	/*@fallthrough@*/
     case RPMQV_PATH:
-    {   const char * s;
-	char * fn;
+    {   char * fn;
 
 	for (s = arg; *s != '\0'; s++)
 	    if (!(*s == '.' || *s == '/'))
@@ -796,14 +921,13 @@ restart:
 	}
 	recOffset = strtoul(myarg, &end, mybase);
 	if ((*end) || (end == arg) || (recOffset == ULONG_MAX)) {
-	    rpmError(RPMERR_QUERY, _("invalid package number: %s\n"), arg);
+	    rpmError(RPMERR_QUERYINFO, _("invalid package number: %s\n"), arg);
 	    return 1;
 	}
 	rpmMessage(RPMMESS_DEBUG, _("package record number: %u\n"), recOffset);
-	/* RPMDBI_PACKAGES */
 	mi = rpmdbInitIterator(rpmdb, RPMDBI_PACKAGES, &recOffset, sizeof(recOffset));
 	if (mi == NULL) {
-	    rpmError(RPMERR_QUERY,
+	    rpmError(RPMERR_QUERYINFO,
 		_("record %u could not be read\n"), recOffset);
 	    retcode = 1;
 	} else {
