@@ -17,7 +17,8 @@
 
 static int manageFile(FD_t *fdp, const char **fnp, int flags,
 		/*@unused@*/ int rc)
-	/*@modifies *fdp, *fnp, fileSystem @*/
+	/*@globals rpmGlobalMacroContext, fileSystem @*/
+	/*@modifies *fdp, *fnp, rpmGlobalMacroContext, fileSystem @*/
 {
     const char *fn;
     FD_t fd;
@@ -55,7 +56,7 @@ static int manageFile(FD_t *fdp, const char **fnp, int flags,
 	if (fnp)
 	    *fnp = fn;
 	*fdp = fdLink(fd, "manageFile return");
-	fdFree(fd, "manageFile return");
+	(void) fdFree(fd, "manageFile return");
 	return 0;
     }
 
@@ -160,6 +161,19 @@ int rpmReSign(rpmResignFlags flags, char * passPhrase, const char ** argv)
 
 	/* Generate the new signatures */
 	if (flags != RESIGN_ADD_SIGNATURE) {
+	    void * uh = NULL;
+	    int_32 uht, uhc;
+
+	    /* Dump the immutable region (if present). */
+	    if (headerGetEntry(sig, RPMTAG_HEADERSIGNATURES, &uht, &uh, &uhc)) {
+		Header nh = headerCopyLoad(uh);
+		sig = headerFree(sig);
+		/*@-nullpass@*/
+		sig = headerLink(nh);
+		/*@=nullpass@*/
+		nh = headerFree(nh);
+	    }
+
 	    (void) headerRemoveEntry(sig, RPMSIGTAG_SIZE);
 	    (void) headerRemoveEntry(sig, RPMSIGTAG_MD5);
 	    (void) headerRemoveEntry(sig, RPMSIGTAG_LEMD5_1);
@@ -177,7 +191,7 @@ int rpmReSign(rpmResignFlags flags, char * passPhrase, const char ** argv)
 	/* Write the lead/signature of the output rpm */
 	strcpy(tmprpm, rpm);
 	strcat(tmprpm, ".XXXXXX");
-	/*@-unrecog@*/ mktemp(tmprpm) /*@=unrecog@*/;
+	(void) /*@-unrecog@*/ mktemp(tmprpm) /*@=unrecog@*/;
 	trpm = tmprpm;
 
 	if (manageFile(&ofd, &trpm, O_WRONLY|O_CREAT|O_TRUNC, 0))
@@ -348,8 +362,9 @@ int rpmCheckSig(rpmCheckSigFlags flags, const char ** argv)
 		      case RPMSIGTAG_PGP5:	/* XXX legacy */
 		      case RPMSIGTAG_PGP:
 			switch (res3) {
-			/* Do not consider these a failure */
 			case RPMSIG_NOKEY:
+			    res2 = 1;
+			    /*@fallthrough@*/
 			case RPMSIG_NOTTRUSTED:
 			{   int offset = 7;
 			    strcat(buffer, "(PGP) ");
@@ -389,6 +404,7 @@ int rpmCheckSig(rpmCheckSigFlags flags, const char ** argv)
 				/*@-compdef@*/
 				strncat(missingKeys, tempKey+7, 8);
 				/*@=compdef@*/
+			    res2 = 1;
 			    break;
 			default:
 			    strcat(buffer, "GPG ");
