@@ -12,7 +12,10 @@
 /** \ingroup rpmcli
  * Should version 3 packages be produced?
  */
+/*@-redecl@*/
+/*@unchecked@*/
 extern int _noDirTokens;
+/*@=redecl@*/
 
 #ifdef __cplusplus
 extern "C" {
@@ -27,8 +30,10 @@ extern "C" {
  */
 struct rpmBuildArguments_s {
     int buildAmount;		/*!< Bit(s) to control operation. */
-/*@null@*/ const char * buildRootOverride; /*!< from --buildroot */
-/*@null@*/ char * targets;	/*!< Target platform(s), comma separated. */
+/*@null@*/
+    const char * buildRootOverride; /*!< from --buildroot */
+/*@null@*/
+    char * targets;		/*!< Target platform(s), comma separated. */
     int force;			/*!< from --force */
     int noBuild;		/*!< from --nobuild */
     int noDeps;			/*!< from --nodeps */
@@ -38,7 +43,8 @@ struct rpmBuildArguments_s {
     int useCatalog;		/*!< from --usecatalog */
     char buildMode;		/*!< Build mode (one of "btBC") */
     char buildChar;		/*!< Build stage (one of "abcilps ") */
-/*@observer@*/ /*@null@*/ const char * rootdir;
+/*@observer@*/ /*@null@*/
+    const char * rootdir;
 };
 
 /** \ingroup rpmcli
@@ -47,10 +53,12 @@ typedef	struct rpmBuildArguments_s *	BTA_t;
 
 /** \ingroup rpmcli
  */
+/*@unchecked@*/
 extern struct rpmBuildArguments_s	rpmBTArgs;
 
 /** \ingroup rpmcli
  */
+/*@unchecked@*/
 extern struct poptOption		rpmBuildPoptTable[];
 
 /*@}*/
@@ -84,7 +92,7 @@ typedef enum rpmQueryFlags_e {
     QUERY_FOR_STATE	= (1 << 24),	/*!< query:  from --state */
     QUERY_FOR_DOCS	= (1 << 25),	/*!< query:  from --docfiles */
     QUERY_FOR_CONFIG	= (1 << 26),	/*!< query:  from --configfiles */
-    QUERY_FOR_DUMPFILES	= (1 << 27),	/*!< query:  from --dump */
+    QUERY_FOR_DUMPFILES	= (1 << 27)	/*!< query:  from --dump */
 } rpmQueryFlags;
 
 /** \ingroup rpmcli
@@ -114,7 +122,7 @@ typedef enum rpmVerifyFlags_e {
     VERIFY_FOR_STATE	= (1 << 24),	/*!< query:  from --state */
     VERIFY_FOR_DOCS	= (1 << 25),	/*!< query:  from --docfiles */
     VERIFY_FOR_CONFIG	= (1 << 26),	/*!< query:  from --configfiles */
-    VERIFY_FOR_DUMPFILES= (1 << 27),	/*!< query:  from --dump */
+    VERIFY_FOR_DUMPFILES= (1 << 27)	/*!< query:  from --dump */
 /*@=enummemuse@*/
 	/* bits 28-30 used in rpmVerifyAttrs */
 } rpmVerifyFlags;
@@ -131,6 +139,7 @@ typedef enum rpmVerifyFlags_e {
  * @param h		header to use for query/verify
  */
 typedef	int (*QVF_t) (QVA_t qva, rpmdb db, Header h)
+	/*@globals fileSystem @*/
 	/*@modifies db, fileSystem @*/;
 
 /** \ingroup rpmcli
@@ -149,6 +158,7 @@ int showMatches(QVA_t qva, /*@only@*/ /*@null@*/ rpmdbMatchIterator mi,
  * @param fp	file handle to use for display
  */
 void rpmDisplayQueryTags(FILE * fp)
+	/*@globals fileSystem @*/
 	/*@modifies *fp, fileSystem @*/;
 
 /** \ingroup rpmcli
@@ -162,7 +172,8 @@ void rpmDisplayQueryTags(FILE * fp)
  */
 int rpmQueryVerify(QVA_t qva, rpmQVSources source, const char * arg,
 		rpmdb db, QVF_t showPackage)
-	/*@modifies db, fileSystem @*/;
+	/*@globals rpmGlobalMacroContext, fileSystem @*/
+	/*@modifies db, rpmGlobalMacroContext, fileSystem @*/;
 
 /** \ingroup rpmcli
  * Display results of package query.
@@ -173,6 +184,7 @@ int rpmQueryVerify(QVA_t qva, rpmQVSources source, const char * arg,
  * @return		0 always
  */
 int showQueryPackage(QVA_t qva, rpmdb db, Header h)
+	/*@globals fileSystem @*/
 	/*@modifies db, fileSystem @*/;
 
 /** \ingroup rpmcli
@@ -184,7 +196,8 @@ int showQueryPackage(QVA_t qva, rpmdb db, Header h)
  * @return		rpmQueryVerify() result, or 1 on rpmdbOpen() failure
  */
 int rpmQuery(QVA_t qva, rpmQVSources source, const char * arg)
-	/*@modifies fileSystem @*/;
+	/*@globals rpmGlobalMacroContext, fileSystem @*/
+	/*@modifies rpmGlobalMacroContext, fileSystem @*/;
 
 /** \ingroup rpmcli
  * Display results of package verify.
@@ -194,7 +207,8 @@ int rpmQuery(QVA_t qva, rpmQVSources source, const char * arg)
  * @return		result of last non-zero verify return
  */
 int showVerifyPackage(QVA_t qva, /*@only@*/ rpmdb db, Header h)
-	/*@modifies db, h, fileSystem @*/;
+	/*@globals rpmGlobalMacroContext, fileSystem @*/
+	/*@modifies db, h, rpmGlobalMacroContext, fileSystem @*/;
 
 /**
  * Check original header digest.
@@ -213,7 +227,8 @@ int rpmVerifyDigest(Header h)
  * @return		rpmQueryVerify() result, or 1 on rpmdbOpen() failure
  */
 int rpmVerify(QVA_t qva, rpmQVSources source, const char *arg)
-	/*@modifies fileSystem @*/;
+	/*@globals rpmGlobalMacroContext, fileSystem @*/
+	/*@modifies rpmGlobalMacroContext, fileSystem @*/;
 
 /** \ingroup rpmcli
  * Describe query/verify command line request.
@@ -223,31 +238,39 @@ struct rpmQVArguments_s {
     int 	qva_sourceCount;/*!< Exclusive check (>1 is error). */
     rpmQueryFlags qva_flags;	/*!< Bit(s) to control operation. */
     rpmfileAttrs qva_fflags;	/*!< Bit(s) to filter on attribute. */
-/*@unused@*/ int qva_verbose;	/*!< (unused) */
-/*@only@*/ /*@null@*/ const char * qva_queryFormat; /*!< Format for headerSprintf(). */
-/*@observer@*/ /*@null@*/ const char * qva_prefix; /*!< Path to top of install tree. */
+/*@unused@*/
+    int qva_verbose;		/*!< (unused) */
+/*@only@*/ /*@null@*/
+    const char * qva_queryFormat; /*!< Format for headerSprintf(). */
+/*@observer@*/ /*@null@*/
+    const char * qva_prefix;	/*!< Path to top of install tree. */
     char	qva_mode;	/*!< 'q' is query, 'v' is verify mode. */
     char	qva_char;	/*!< (unused) always ' ' */
 };
 
 /** \ingroup rpmcli
  */
+/*@unchecked@*/
 extern struct rpmQVArguments_s rpmQVArgs;
 
 /** \ingroup rpmcli
  */
+/*@unchecked@*/
 extern struct poptOption rpmQVSourcePoptTable[];
 
 /** \ingroup rpmcli
  */
+/*@unchecked@*/
 extern int specedit;
 
 /** \ingroup rpmcli
  */
+/*@unchecked@*/
 extern struct poptOption rpmQueryPoptTable[];
 
 /** \ingroup rpmcli
  */
+/*@unchecked@*/
 extern struct poptOption rpmVerifyPoptTable[];
 
 /*@}*/
@@ -272,7 +295,8 @@ int rpmInstall(/*@null@*/ const char * rootdir,
 		rpmInstallInterfaceFlags interfaceFlags,
 		rpmprobFilterFlags probFilter,
 		/*@null@*/ rpmRelocation * relocations)
-	/*@modifies *relocations, fileSystem @*/;
+	/*@globals rpmGlobalMacroContext, fileSystem @*/
+	/*@modifies *relocations, rpmGlobalMacroContext, fileSystem @*/;
 
 /** \ingroup rpmcli
  * Install source rpm package.
@@ -285,6 +309,7 @@ int rpmInstall(/*@null@*/ const char * rootdir,
 int rpmInstallSource(const char * rootdir, const char * arg,
 		/*@null@*/ /*@out@*/ const char ** specFile,
 		/*@null@*/ /*@out@*/ char ** cookie)
+	/*@globals fileSystem @*/
 	/*@modifies *specFile, *cookie, fileSystem @*/;
 
 /** \ingroup rpmcli
@@ -298,7 +323,8 @@ int rpmInstallSource(const char * rootdir, const char * arg,
 int rpmErase(/*@null@*/ const char * rootdir, /*@null@*/ const char ** argv,
 		rpmtransFlags transFlags, 
 		rpmEraseInterfaceFlags interfaceFlags)
-	/*@modifies fileSystem @*/;
+	/*@globals rpmGlobalMacroContext, fileSystem @*/
+	/*@modifies rpmGlobalMacroContext, fileSystem @*/;
 
 /** \ingroup rpmcli
  * Describe database command line requests.
@@ -308,13 +334,16 @@ struct rpmInstallArguments_s {
     rpmprobFilterFlags probFilter;
     rpmInstallInterfaceFlags installInterfaceFlags;
     rpmEraseInterfaceFlags eraseInterfaceFlags;
-/*@only@*/ /*@null@*/ rpmRelocation * relocations;
+/*@only@*/ /*@null@*/
+    rpmRelocation * relocations;
     int numRelocations;
     int noDeps;
     int incldocs;
-/*@null@*/ const char * prefix;
-/*@observer@*/ /*@null@*/ const char * rootdir;
-    int_32 rbtid;		/*!< from --rollback */
+/*@null@*/
+    const char * prefix;
+/*@observer@*/ /*@null@*/
+    const char * rootdir;
+    uint_32 rbtid;		/*!< from --rollback */
 };
 
 /**
@@ -322,10 +351,14 @@ struct rpmInstallArguments_s {
  */
 typedef /*@abstract@*/ struct IDT_s {
     unsigned int instance;	/*!< installed package transaction id. */
-/*@owned@*/ /*@null@*/ const char * key; /*! removed package file name. */
+/*@owned@*/ /*@null@*/
+    const char * key;		/*! removed package file name. */
     Header h;			/*!< removed package header. */
+    const char * n;		/*!< package name. */
+    const char * v;		/*!< package version. */
+    const char * r;		/*!< package release. */
     union {
-	int_32 i32;		/*!< install/remove transaction id */
+	uint_32 u32;		/*!< install/remove transaction id */
     } val;
 } * IDT;
 
@@ -337,7 +370,8 @@ typedef /*@abstract@*/ struct IDTindex_s {
     int size;			/*!< size of id index element. */
     int alloced;		/*!< current number of elements allocated. */
     int nidt;			/*!< current number of elements initialized. */
-/*@only@*/ /*@null@*/ IDT idt;	/*!< id index elements. */
+/*@only@*/ /*@null@*/
+    IDT idt;			/*!< id index elements. */
 } * IDTX;
 
 /**
@@ -388,6 +422,7 @@ typedef /*@abstract@*/ struct IDTindex_s {
  * @return 		id index
  */
 /*@only@*/ /*@null@*/ IDTX IDTXglob(const char * globstr, rpmTag tag)
+	/*@globals fileSystem @*/
 	/*@modifies fileSystem @*/;
 
 
@@ -410,8 +445,10 @@ typedef /*@abstract@*/ struct IDTindex_s {
 		const unsigned long total,
 		/*@null@*/ const void * pkgKey,
 		/*@null@*/ void * data)
-	/*@modifies fileSystem @*/;
+	/*@globals internalState, fileSystem @*/
+	/*@modifies internalState, fileSystem @*/;
 
+/*@unchecked@*/
 extern int packagesTotal;
 
 /** \ingroup rpmcli
@@ -420,14 +457,17 @@ extern int packagesTotal;
  */
 int rpmRollback(struct rpmInstallArguments_s * ia,
 		/*@null@*/ const char ** argv)
-	/*@modifies fileSystem @*/;
+	/*@globals rpmGlobalMacroContext, fileSystem @*/
+	/*@modifies rpmGlobalMacroContext, fileSystem @*/;
 
 /** \ingroup rpmcli
  */
+/*@unchecked@*/
 extern struct rpmInstallArguments_s rpmIArgs;
 
 /** \ingroup rpmcli
  */
+/*@unchecked@*/
 extern struct poptOption rpmInstallPoptTable[];
 
 /*@}*/
@@ -447,10 +487,12 @@ struct rpmDatabaseArguments_s {
 
 /** \ingroup rpmcli
  */
+/*@unchecked@*/
 extern struct rpmDatabaseArguments_s rpmDBArgs;
 
 /** \ingroup rpmcli
  */
+/*@unchecked@*/
 extern struct poptOption rpmDatabasePoptTable[];
 
 /*@}*/
@@ -478,7 +520,8 @@ typedef enum rpmCheckSigFlags_e {
  * @return		0 on success
  */
 int rpmCheckSig(rpmCheckSigFlags flags, /*@null@*/ const char ** argv)
-	/*@modifies fileSystem @*/;
+	/*@globals rpmGlobalMacroContext, fileSystem @*/
+	/*@modifies rpmGlobalMacroContext, fileSystem @*/;
 
 /** \ingroup rpmcli
  * Bit(s) to control rpmReSign() operation.
@@ -499,7 +542,8 @@ typedef enum rpmResignFlags_e {
  */
 int rpmReSign(rpmResignFlags add, char * passPhrase,
 		/*@null@*/ const char ** argv)
-	/*@modifies fileSystem @*/;
+	/*@globals rpmGlobalMacroContext, fileSystem @*/
+	/*@modifies rpmGlobalMacroContext, fileSystem @*/;
 
 /** \ingroup rpmcli
  * Describe signature command line request.
@@ -513,10 +557,12 @@ struct rpmSignArguments_s {
 
 /** \ingroup rpmcli
  */
+/*@unchecked@*/
 extern struct rpmSignArguments_s rpmKArgs;
 
 /** \ingroup rpmcli
  */
+/*@unchecked@*/
 extern struct poptOption rpmSignPoptTable[];
 
 /*@}*/
