@@ -18,16 +18,21 @@
 /*@access IDTX @*/
 /*@access IDT @*/
 
+/*@unchecked@*/
 static int hashesPrinted = 0;
 
+/*@unchecked@*/
 int packagesTotal = 0;
+/*@unchecked@*/
 static int progressTotal = 0;
+/*@unchecked@*/
 static int progressCurrent = 0;
 
 /**
  */
 static void printHash(const unsigned long amount, const unsigned long total)
-	/*@modifies fileSystem @*/
+	/*@globals hashesPrinted, progressCurrent, fileSystem @*/
+	/*@modifies hashesPrinted, progressCurrent, fileSystem @*/
 {
     int hashesNeeded;
     int hashesTotal = 50;
@@ -42,7 +47,7 @@ static void printHash(const unsigned long amount, const unsigned long total)
 		int i;
 		for (i = 0; i < hashesPrinted; i++) (void) putchar ('#');
 		for (; i < hashesTotal; i++) (void) putchar (' ');
-		printf ("(%3d%%)",
+		fprintf(stdout, "(%3d%%)",
 			(int)(100 * (total ? (((float) amount) / total) : 1)));
 		for (i = 0; i < (hashesTotal + 6); i++) (void) putchar ('\b');
 	    } else
@@ -56,9 +61,12 @@ static void printHash(const unsigned long amount, const unsigned long total)
 	if (hashesPrinted == hashesTotal) {
 	    int i;
 	    progressCurrent++;
-	    for (i = 1; i < hashesPrinted; i++) (void) putchar ('#');
-	    printf (" [%3d%%]\n", (int)(100 * (progressTotal ?
+	    if (isatty(STDOUT_FILENO)) {
+	        for (i = 1; i < hashesPrinted; i++) (void) putchar ('#');
+		fprintf(stdout, " [%3d%%]", (int)(100 * (progressTotal ?
 			(((float) progressCurrent) / progressTotal) : 1)));
+	    }
+	    fprintf(stdout, "\n");
 	}
 	(void) fflush(stdout);
     }
@@ -70,7 +78,10 @@ void * rpmShowProgress(/*@null@*/ const void * arg,
 			const unsigned long total,
 			/*@null@*/ const void * pkgKey,
 			/*@null@*/ void * data)
-	/*@modifies fileSystem @*/
+	/*@globals hashesPrinted, progressCurrent, progressTotal,
+		fileSystem @*/
+	/*@modifies hashesPrinted, progressCurrent, progressTotal,
+		fileSystem @*/
 {
     /*@-castexpose@*/
     Header h = (Header) arg;
@@ -108,6 +119,8 @@ void * rpmShowProgress(/*@null@*/ const void * arg,
 				rpmTagTable, rpmHeaderFormats, NULL);
 	    if (isatty (STDOUT_FILENO))
 		fprintf(stdout, "%4d:%-23.23s", progressCurrent + 1, s);
+	    else
+		fprintf(stdout, "%-28.28s", s);
 	    (void) fflush(stdout);
 	    s = _free(s);
 	} else {
@@ -139,7 +152,7 @@ void * rpmShowProgress(/*@null@*/ const void * arg,
 	if (flags & INSTALL_HASH)
 	    fprintf(stdout, "%-28s", _("Preparing..."));
 	else
-	    printf("%s\n", _("Preparing packages for installation..."));
+	    fprintf(stdout, "%s\n", _("Preparing packages for installation..."));
 	(void) fflush(stdout);
 	break;
 
@@ -216,6 +229,7 @@ int rpmInstall(const char * rootdir, const char ** fileArgv,
     }
 
     /* Build fully globbed list of arguments in argv[argc]. */
+    /*@-branchstate@*/
     /*@-temptrans@*/
     for (eiu->fnp = fileArgv; *eiu->fnp != NULL; eiu->fnp++) {
     /*@=temptrans@*/
@@ -228,6 +242,7 @@ int rpmInstall(const char * rootdir, const char ** fileArgv,
 	eiu->argc += ac;
 	eiu->argv[eiu->argc] = NULL;
     }
+    /*@=branchstate@*/
     av = _free(av);	ac = 0;
 
 restart:
@@ -260,7 +275,7 @@ restart:
 
 	    {	char tfnbuf[64];
 		strcpy(tfnbuf, "rpm-xfer.XXXXXX");
-		/*@-unrecog@*/ mktemp(tfnbuf) /*@=unrecog@*/;
+		(void) mktemp(tfnbuf);
 		tfn = rpmGenPath(rootdir, "%{_tmppath}/", tfnbuf);
 	    }
 
@@ -275,18 +290,18 @@ restart:
 		eiu->numFailed++;
 		eiu->pkgURL[eiu->pkgx] = NULL;
 		tfn = _free(tfn);
-		break;
+		/*@switchbreak@*/ break;
 	    }
 	    eiu->pkgState[eiu->pkgx] = 1;
 	    eiu->pkgURL[eiu->pkgx] = tfn;
 	    eiu->pkgx++;
-	}   break;
+	}   /*@switchbreak@*/ break;
 	case URL_IS_PATH:
 	default:
 	    eiu->pkgURL[eiu->pkgx] = fileURL;
 	    fileURL = NULL;
 	    eiu->pkgx++;
-	    break;
+	    /*@switchbreak@*/ break;
 	}
     }
     fileURL = _free(fileURL);
@@ -396,7 +411,7 @@ restart:
 		count = rpmdbGetIteratorCount(mi);
 		while ((oldH = rpmdbNextIterator(mi)) != NULL) {
 		    if (rpmVersionCompare(oldH, eiu->h) < 0)
-			continue;
+			/*@innercontinue@*/ continue;
 		    /* same or newer package already installed */
 		    count = 0;
 		    /*@innerbreak@*/ break;
@@ -421,20 +436,20 @@ restart:
 	    case 0:
 		rpmMessage(RPMMESS_DEBUG, "\tadded binary package [%d]\n",
 			eiu->numRPMS);
-		break;
+		/*@switchbreak@*/ break;
 	    case 1:
 		rpmMessage(RPMMESS_ERROR,
 			    _("error reading from file %s\n"), *eiu->fnp);
 		eiu->numFailed++;
 		goto exit;
-		/*@notreached@*/ break;
+		/*@notreached@*/ /*@switchbreak@*/ break;
 	    case 2:
 		rpmMessage(RPMMESS_ERROR,
 			    _("file %s requires a newer version of RPM\n"),
 			    *eiu->fnp);
 		eiu->numFailed++;
 		goto exit;
-		/*@notreached@*/ break;
+		/*@notreached@*/ /*@switchbreak@*/ break;
 	    }
 
 	    eiu->numRPMS++;
@@ -492,6 +507,7 @@ restart:
 	    stopInstall = 1;
 	}
 
+	/*@-branchstate@*/
 	if (!stopInstall && conflicts) {
 	    rpmMessage(RPMMESS_ERROR, _("failed dependencies:\n"));
 	    printDepProblems(stderr, conflicts, numConflicts);
@@ -499,6 +515,7 @@ restart:
 	    eiu->numFailed = eiu->numPkgs;
 	    stopInstall = 1;
 	}
+	/*@=branchstate@*/
     }
 
     if (eiu->numRPMS && !(interfaceFlags & INSTALL_NOORDER)) {
@@ -637,6 +654,7 @@ int rpmErase(const char * rootdir, const char ** argv,
 	    stopUninstall = 1;
 	}
 
+	/*@-branchstate@*/
 	if (!stopUninstall && conflicts) {
 	    rpmMessage(RPMMESS_ERROR, _("removing these packages would break "
 			      "dependencies:\n"));
@@ -645,6 +663,7 @@ int rpmErase(const char * rootdir, const char ** argv,
 	    numFailed += numPackages;
 	    stopUninstall = 1;
 	}
+	/*@=branchstate@*/
     }
 
     if (!stopUninstall) {
@@ -697,6 +716,7 @@ int rpmInstallSource(const char * rootdir, const char * arg,
     return rc;
 }
 
+/*@unchecked@*/
 static int reverse = -1;
 
 /**
@@ -705,8 +725,14 @@ static int IDTintcmp(const void * a, const void * b)
 	/*@*/
 {
     /*@-castexpose@*/
-    return ( reverse * (((IDT)a)->val.i32 - ((IDT)b)->val.i32) );
+    IDT ap = (IDT)a;
+    IDT bp = (IDT)b;
     /*@=castexpose@*/
+    int rc = ((int)ap->val.u32 - (int)bp->val.u32);
+
+    if (rc)
+	return ( reverse * rc );
+    return ( strcmp(ap->n, bp->n) );
 }
 
 IDTX IDTXfree(IDTX idtx)
@@ -765,6 +791,7 @@ IDTX IDTXload(rpmdb db, rpmTag tag)
     Header h;
 
     mi = rpmdbInitIterator(db, tag, NULL, 0);
+    /*@-branchstate@*/
     while ((h = rpmdbNextIterator(mi)) != NULL) {
 	rpmTagType type = RPM_NULL_TYPE;
 	int_32 count = 0;
@@ -787,13 +814,15 @@ IDTX IDTXload(rpmdb db, rpmTag tag)
 	    /*@-nullderef@*/
 	    idt = idtx->idt + idtx->nidt;
 	    /*@=nullderef@*/
-	    idt->h = NULL;
+	    idt->h = headerLink(h);
+	    (void) headerNVR(idt->h, &idt->n, &idt->v, &idt->r);
 	    idt->key = NULL;
 	    idt->instance = rpmdbGetIteratorOffset(mi);
-	    idt->val.i32 = *tidp;
+	    idt->val.u32 = *tidp;
 	}
 	idtx->nidt++;
     }
+    /*@=branchstate@*/
     mi = rpmdbFreeIterator(mi);
 
     return IDTXsort(idtx);
@@ -838,6 +867,7 @@ IDTX IDTXglob(const char * globstr, rpmTag tag)
 	}
 
 	tidp = NULL;
+	/*@-branchstate@*/
 	if (hge(h, tag, &type, (void **) &tidp, &count) && tidp) {
 
 	    idtx = IDTXgrow(idtx, 1);
@@ -849,13 +879,14 @@ IDTX IDTXglob(const char * globstr, rpmTag tag)
 	    {	IDT idt;
 		idt = idtx->idt + idtx->nidt;
 		idt->h = headerLink(h);
-		idt->key = av[i];
-		av[i] = NULL;
+		(void) headerNVR(idt->h, &idt->n, &idt->v, &idt->r);
+		idt->key = xstrdup(av[i]);
 		idt->instance = 0;
-		idt->val.i32 = *tidp;
+		idt->val.u32 = *tidp;
 	    }
 	    idtx->nidt++;
 	}
+	/*@=branchstate@*/
 
 	h = headerFree(h);
 	(void) Fclose(fd);
@@ -865,31 +896,203 @@ IDTX IDTXglob(const char * globstr, rpmTag tag)
 	av[i] = _free(av[i]);
     av = _free(av);	ac = 0;
 
-    return idtx;
+    return IDTXsort(idtx);
 }
 
-int rpmRollback(/*@unused@*/ struct rpmInstallArguments_s * ia,
-		/*@unused@*/ const char ** argv)
+int rpmRollback(struct rpmInstallArguments_s * ia, const char ** argv)
 {
     rpmdb db = NULL;
     rpmTransactionSet ts = NULL;
+    rpmDependencyConflict conflicts = NULL;
+    int numConflicts = 0;
+    rpmProblemSet probs = NULL;
     IDTX itids = NULL;
-    const char * globstr = rpmExpand("%{_repackage_dir}/*.rpm", NULL);
     IDTX rtids = NULL;
+    unsigned thistid = 0xffffffff;
+    unsigned prevtid;
+    time_t tid;
+    IDT rp;
+    int nrids = 0;
+    IDT ip;
+    int niids = 0;
+    int packagesIn;
+    int packagesOut;
     int rc;
+    int i;
+    int ifmask= (INSTALL_UPGRADE|INSTALL_FRESHEN|INSTALL_INSTALL|INSTALL_ERASE);
+
+    if (argv != NULL && *argv != NULL) {
+	rc = -1;
+	goto exit;
+    }
 
     rc = rpmdbOpen(ia->rootdir, &db, O_RDWR, 0644);
-
-    ts = rpmtransCreateSet(db, ia->rootdir);
+    if (rc != 0)
+	goto exit;
 
     itids = IDTXload(db, RPMTAG_INSTALLTID);
-    rtids = IDTXglob(globstr, RPMTAG_REMOVETID);
+    if (itids != NULL) {
+	ip = itids->idt;
+	niids = itids->nidt;
+    } else {
+	ip = NULL;
+	niids = 0;
+    }
+
+    {	const char * globstr = rpmExpand("%{_repackage_dir}/*.rpm", NULL);
+	if (globstr == NULL || *globstr == '%') {
+	    globstr = _free(globstr);
+	    rc = -1;
+	    goto exit;
+	}
+	rtids = IDTXglob(globstr, RPMTAG_REMOVETID);
+	if (rtids != NULL) {
+	    rp = rtids->idt;
+	    nrids = rtids->nidt;
+	} else {
+	    rp = NULL;
+	    nrids = 0;
+	}
+	globstr = _free(globstr);
+    }
+
+    /* Run transactions until rollback goal is achieved. */
+    do {
+	prevtid = thistid;
+	rc = 0;
+	packagesTotal = 0;
+	packagesIn = 0;
+	packagesOut = 0;
+	ia->installInterfaceFlags &= ~ifmask;
+
+	/* Find larger of the remaining install/erase transaction id's. */
+	thistid = 0;
+	if (ip != NULL && ip->val.u32 > thistid)
+	    thistid = ip->val.u32;
+	if (rp != NULL && rp->val.u32 > thistid)
+	    thistid = rp->val.u32;
+
+	/* If we've achieved the rollback goal, then we're done. */
+	if (thistid == 0 || thistid < ia->rbtid)
+	    break;
+
+	ts = rpmtransCreateSet(db, ia->rootdir);
+
+	/* Install the previously erased packages for this transaction. */
+	while (rp != NULL && rp->val.u32 == thistid) {
+
+	    rpmMessage(RPMMESS_DEBUG, "\t+++ %s-%s-%s\t(from %s)\n", rp->n, rp->v, rp->r, basename(rp->key));
+
+	    if (!(ia->installInterfaceFlags & ifmask))
+		ia->installInterfaceFlags |= INSTALL_INSTALL;
+
+	    rc = rpmtransAddPackage(ts, rp->h, NULL, rp->key,
+			(ia->installInterfaceFlags & INSTALL_UPGRADE) != 0,
+			ia->relocations);
+	    if (rc != 0)
+		goto exit;
+
+	    packagesTotal++;
+	    packagesIn++;
+
+#ifdef	NOTYET
+	    rp->h = headerFree(rp->h);
+#endif
+	    nrids--;
+	    if (nrids > 0)
+		rp++;
+	    else
+		rp = NULL;
+	}
+
+	/*
+	 * XXX OK, let's prevent disaster right here, as rollbacks will merrily
+	 * XXX erase everything in order to achieve the desired goal.
+	 */
+	if (packagesIn == 0)
+	    break;
+
+	/* Erase the previously installed packages for this transaction. */
+	while (ip != NULL && ip->val.u32 == thistid) {
+
+	    rpmMessage(RPMMESS_DEBUG,
+			"\t--- %s-%s-%s\t(from rpmdb instance #%u)\n", ip->n, ip->v, ip->r, ip->instance);
+
+	    if (!(ia->installInterfaceFlags & ifmask))
+		ia->installInterfaceFlags |= INSTALL_ERASE;
+
+	    rc = rpmtransRemovePackage(ts, ip->instance);
+	    if (rc != 0)
+		goto exit;
+
+	    packagesTotal++;
+	    packagesOut++;
+
+#ifdef	NOTYET
+	    ip->instance = 0;
+#endif
+	    niids--;
+	    if (niids > 0)
+		ip++;
+	    else
+		ip = NULL;
+	}
+
+	/* Anything to do? */
+	if (packagesTotal <= 0)
+	    break;
+
+	tid = (time_t)thistid;
+	rpmMessage(RPMMESS_DEBUG, _("rollback (+%d,-%d) packages to %s"),
+			packagesIn, packagesOut, ctime(&tid));
+
+	conflicts = NULL;
+	numConflicts = 0;
+	rc = rpmdepCheck(ts, &conflicts, &numConflicts);
+	if (rc != 0) {
+	    rpmMessage(RPMMESS_ERROR, _("failed dependencies:\n"));
+	    printDepProblems(stderr, conflicts, numConflicts);
+	    conflicts = rpmdepFreeConflicts(conflicts, numConflicts);
+	    goto exit;
+	}
+
+	rc = rpmdepOrder(ts);
+	if (rc != 0)
+	    goto exit;
+
+	probs = NULL;
+	rc = rpmRunTransactions(ts,  rpmShowProgress,
+		(void *) ((long)ia->installInterfaceFlags),
+		NULL, &probs, ia->transFlags,
+		(ia->probFilter|RPMPROB_FILTER_OLDPACKAGE));
+	if (rc > 0) {
+	    rpmProblemSetPrint(stderr, probs);
+	    if (probs != NULL) rpmProblemSetFree(probs);
+	    probs = NULL;
+	}
+	if (rc)
+	    goto exit;
+
+	ts = rpmtransFree(ts);
+
+	/* Clean up after successful rollback. */
+	if (!rpmIsDebug())
+	for (i = 0; i < rtids->nidt; i++) {
+	    IDT rrp = rtids->idt + i;
+	    if (rrp->val.u32 != thistid)
+		continue;
+	    (void) unlink(rrp->key);
+	}
+
+    } while (1);
+
+exit:
+    ts = rpmtransFree(ts);
+
+    if (db != NULL) (void) rpmdbClose(db);
 
-    globstr = _free(globstr);
     rtids = IDTXfree(rtids);
     itids = IDTXfree(itids);
 
-    ts =  rpmtransFree(ts);
-
-    return 0;
+    return rc;
 }
