@@ -7,7 +7,7 @@
 #include <rpmlib.h>
 #include "debug.h"
 
-static struct rpmlibProvides {
+static struct rpmlibProvides_s {
 /*@observer@*/ /*@null@*/ const char * featureName;
 /*@observer@*/ /*@null@*/ const char * featureEVR;
     int featureFlags;
@@ -18,28 +18,31 @@ static struct rpmlibProvides {
     N_("PreReq:, Provides:, and Obsoletes: dependencies support versions.") },
     { "rpmlib(CompressedFileNames)",	"3.0.4-1",
 	(RPMSENSE_RPMLIB|RPMSENSE_EQUAL),
-    N_("file names stored as (dirName,baseName,dirIndex) tuple, not as path.")},
+    N_("file name(s) are stored as (dirName,baseName,dirIndex) tuple, not as path.")},
     { "rpmlib(PayloadIsBzip2)",		"3.0.5-1",
 	(RPMSENSE_RPMLIB|RPMSENSE_EQUAL),
-    N_("package payload compressed using bzip2.") },
+    N_("package payload is compressed using bzip2.") },
     { "rpmlib(PayloadFilesHavePrefix)",	"4.0-1",
 	(RPMSENSE_RPMLIB|RPMSENSE_EQUAL),
-    N_("package payload files have \"./\" prefix.") },
+    N_("package payload file(s) have \"./\" prefix.") },
     { "rpmlib(ExplicitPackageProvide)",	"4.0-1",
 	(RPMSENSE_RPMLIB|RPMSENSE_EQUAL),
-    N_("package name-version-release not implicitly provided.") },
+    N_("package name-version-release is not implicitly provided.") },
     { "rpmlib(HeaderLoadSortsTags)",    "4.0.1-1",
 	(                RPMSENSE_EQUAL),
     N_("header tags are always sorted after being loaded.") },
     { "rpmlib(ScriptletInterpreterArgs)",    "4.0.3-1",
 	(                RPMSENSE_EQUAL),
     N_("the scriptlet interpreter can use arguments from header.") },
+    { "rpmlib(PartialHardlinkSets)",    "4.0.4-1",
+	(                RPMSENSE_EQUAL),
+    N_("a hardlink file set may be installed without being complete.") },
     { NULL,				NULL, 0,	NULL }
 };
 
 void rpmShowRpmlibProvides(FILE * fp)
 {
-    const struct rpmlibProvides * rlp;
+    const struct rpmlibProvides_s * rlp;
 
     for (rlp = rpmlibProvides; rlp->featureName != NULL; rlp++) {
 	fprintf(fp, "    %s", rlp->featureName);
@@ -54,7 +57,7 @@ void rpmShowRpmlibProvides(FILE * fp)
 int rpmCheckRpmlibProvides(const char * keyName, const char * keyEVR,
 	int keyFlags)
 {
-    const struct rpmlibProvides * rlp;
+    const struct rpmlibProvides_s * rlp;
     int rc = 0;
 
     for (rlp = rpmlibProvides; rlp->featureName != NULL; rlp++) {
@@ -87,20 +90,28 @@ int rpmGetRpmlibProvides(const char *** provNames, int ** provFlags,
         versions[n] = rpmlibProvides[n].featureEVR;
     }
     
+    /*@-branchstate@*/
     if (provNames)
 	*provNames = names;
     else
 	names = _free(names);
+    /*@=branchstate@*/
 
+    /*@-branchstate@*/
     if (provFlags)
 	*provFlags = flags;
     else
 	flags = _free(flags);
+    /*@=branchstate@*/
 
+    /*@-branchstate@*/
     if (provVersions)
 	*provVersions = versions;
     else
 	versions = _free(versions);
+    /*@=branchstate@*/
 
+    /*@-compmempass@*/ /* FIX: rpmlibProvides[] reachable */
     return n;
+    /*@=compmempass@*/
 }
