@@ -13,10 +13,7 @@
 
 #include "system.h"
 
-#if HAVE_ASM_BYTEORDER_H
-#include <asm/byteorder.h>
-#endif
-
+#include "rpmio_internal.h"
 #include <rpmlib.h>
 #include <rpmmacro.h>	/* XXX for rpmGetPath() */
 
@@ -29,8 +26,6 @@
 /*@access Header@*/		/* XXX compared with NULL */
 /*@access FD_t@*/		/* XXX compared with NULL */
 
-typedef	unsigned char byte;
-
 typedef int (*md5func)(const char * fn, /*@out@*/ byte * digest);
 
 int rpmLookupSignatureType(int action)
@@ -48,8 +43,8 @@ int rpmLookupSignatureType(int action)
     case RPMLOOKUPSIG_QUERY:
 	if (disabled)
 	    break;	/* Disabled */
-      { const char *name = rpmExpand("%{_signature}", NULL);
-	if (!(name && *name != '%'))
+      { const char *name = rpmExpand("%{?_signature}", NULL);
+	if (!(name && *name != '\0'))
 	    rc = 0;
 	else if (!xstrcasecmp(name, "none"))
 	    rc = 0;
@@ -72,18 +67,18 @@ int rpmLookupSignatureType(int action)
 
 const char * rpmDetectPGPVersion(pgpVersion * pgpVer)
 {
-    /* Actually this should support having more then one pgp version. */ 
+    /* Actually this should support having more then one pgp version. */
     /* At the moment only one version is possible since we only       */
     /* have one %_pgpbin and one %_pgp_path.                          */
 
     static pgpVersion saved_pgp_version = PGP_UNKNOWN;
-    const char *pgpbin = rpmGetPath("%{_pgpbin}", NULL);
+    const char *pgpbin = rpmGetPath("%{?_pgpbin}", NULL);
 
     if (saved_pgp_version == PGP_UNKNOWN) {
 	char *pgpvbin;
 	struct stat st;
-	
-	if (!(pgpbin && pgpbin[0] != '%')) {
+
+	if (!(pgpbin && pgpbin[0] != '\0')) {
 	  pgpbin = _free(pgpbin);
 	  saved_pgp_version = -1;
 	  return NULL;
@@ -114,6 +109,7 @@ const char * rpmDetectPGPVersion(pgpVersion * pgpVer)
  * @return 			rpmRC return code
  */
 static inline rpmRC checkSize(FD_t fd, int siglen, int pad, int datalen)
+	/*@globals fileSystem @*/
 	/*@modifies fileSystem @*/
 {
     struct stat st;
@@ -131,11 +127,11 @@ static inline rpmRC checkSize(FD_t fd, int siglen, int pad, int datalen)
     rc = (((sizeof(struct rpmlead) + siglen + pad + datalen) - st.st_size)
 	? RPMRC_BADSIZE : RPMRC_OK);
 
-    rpmMessage((rc == RPMRC_OK ? RPMMESS_DEBUG : RPMMESS_WARNING),
+    rpmMessage((rc == RPMRC_OK ? RPMMESS_DEBUG : RPMMESS_DEBUG),
 	_("Expected size: %12d = lead(%d)+sigs(%d)+pad(%d)+data(%d)\n"),
 		(int)sizeof(struct rpmlead)+siglen+pad+datalen,
 		(int)sizeof(struct rpmlead), siglen, pad, datalen);
-    rpmMessage((rc == RPMRC_OK ? RPMMESS_DEBUG : RPMMESS_WARNING),
+    rpmMessage((rc == RPMRC_OK ? RPMMESS_DEBUG : RPMMESS_DEBUG),
 	_("  Actual size: %12d\n"), (int)st.st_size);
 
     return rc;
@@ -213,10 +209,10 @@ rpmRC rpmReadSignature(FD_t fd, Header * headerp, sigType sig_type)
 
 int rpmWriteSignature(FD_t fd, Header h)
 {
-    static byte buf[8] = "\000\000\000\000\000\000\000\000";
+    static byte buf[8] = { 0, 0, 0, 0, 0, 0, 0, 0 };
     int sigSize, pad;
     int rc;
-    
+
     rc = headerWrite(fd, h, HEADER_MAGIC_YES);
     if (rc)
 	return rc;
@@ -244,21 +240,27 @@ Header rpmFreeSignature(Header h)
 
 static int makePGPSignature(const char * file, /*@out@*/ void ** sig,
 		/*@out@*/ int_32 * size, /*@null@*/ const char * passPhrase)
-	/*@modifies *sig, *size, fileSystem @*/
+	/*@globals rpmGlobalMacroContext, fileSystem @*/
+	/*@modifies *sig, *size, rpmGlobalMacroContext, fileSystem @*/
 {
     char * sigfile = alloca(1024);
     int pid, status;
     int inpipe[2];
     struct stat st;
+    const char * cmd;
+    char *const *av;
+    int rc;
 
     (void) stpcpy( stpcpy(sigfile, file), ".sig");
 
+    addMacro(NULL, "__plaintext_filename", NULL, file, -1);
+    addMacro(NULL, "__signature_filename", NULL, sigfile, -1);
+
     inpipe[0] = inpipe[1] = 0;
     (void) pipe(inpipe);
-    
+
     if (!(pid = fork())) {
-	const char *pgp_path = rpmExpand("%{_pgp_path}", NULL);
-	const char *name = rpmExpand("+myname=\"%{_pgp_name}\"", NULL);
+	const char *pgp_path = rpmExpand("%{?_pgp_path}", NULL);
 	const char *path;
 	pgpVersion pgpVer;
 
@@ -267,7 +269,7 @@ static int makePGPSignature(const char * file, /*@out@*/ void ** sig,
 	(void) close(inpipe[1]);
 
 	(void) dosetenv("PGPPASSFD", "3", 1);
-	if (pgp_path && *pgp_path != '%')
+	if (pgp_path && *pgp_path != '\0')
 	    (void) dosetenv("PGPPATH", pgp_path, 1);
 
 	/* dosetenv("PGPPASS", passPhrase, 1); */
@@ -275,23 +277,31 @@ static int makePGPSignature(const char * file, /*@out@*/ void ** sig,
 	if ((path = rpmDetectPGPVersion(&pgpVer)) != NULL) {
 	    switch(pgpVer) {
 	    case PGP_2:
-		(void) execlp(path, "pgp", "+batchmode=on", "+verbose=0", "+armor=off",
-		    name, "-sb", file, sigfile, NULL);
+		cmd = rpmExpand("%{?__pgp_sign_cmd}", NULL);
+		rc = poptParseArgvString(cmd, NULL, (const char ***)&av);
+		if (!rc)
+		    rc = execve(av[0], av+1, environ);
 		break;
 	    case PGP_5:
-		(void) execlp(path,"pgps", "+batchmode=on", "+verbose=0", "+armor=off",
-		    name, "-b", file, "-o", sigfile, NULL);
+		cmd = rpmExpand("%{?__pgp5_sign_cmd}", NULL);
+		rc = poptParseArgvString(cmd, NULL, (const char ***)&av);
+		if (!rc)
+		    rc = execve(av[0], av+1, environ);
 		break;
 	    case PGP_UNKNOWN:
 	    case PGP_NOTDETECTED:
+		errno = ENOENT;
 		break;
 	    }
 	}
-	rpmError(RPMERR_EXEC, _("Couldn't exec pgp (%s)\n"),
-			(path ? path : NULL));
+	rpmError(RPMERR_EXEC, _("Could not exec %s: %s\n"), "pgp",
+			strerror(errno));
 	_exit(RPMERR_EXEC);
     }
 
+    delMacro(NULL, "__plaintext_filename");
+    delMacro(NULL, "__signature_filename");
+
     (void) close(inpipe[0]);
     if (passPhrase)
 	(void) write(inpipe[1], passPhrase, strlen(passPhrase));
@@ -314,9 +324,9 @@ static int makePGPSignature(const char * file, /*@out@*/ void ** sig,
     *size = st.st_size;
     rpmMessage(RPMMESS_DEBUG, _("PGP sig size: %d\n"), *size);
     *sig = xmalloc(*size);
-    
+
     {	FD_t fd;
-	int rc = 0;
+	rc = 0;
 	fd = Fopen(sigfile, "r.fdio");
 	if (fd != NULL && !Ferror(fd)) {
 	    rc = timedRead(fd, *sig, *size);
@@ -331,7 +341,7 @@ static int makePGPSignature(const char * file, /*@out@*/ void ** sig,
     }
 
     rpmMessage(RPMMESS_DEBUG, _("Got %d bytes of PGP sig\n"), *size);
-    
+
     return 0;
 }
 
@@ -342,37 +352,49 @@ static int makePGPSignature(const char * file, /*@out@*/ void ** sig,
  */
 static int makeGPGSignature(const char * file, /*@out@*/ void ** sig,
 		/*@out@*/ int_32 * size, /*@null@*/ const char * passPhrase)
-	/*@modifies *sig, *size, fileSystem @*/
+	/*@globals rpmGlobalMacroContext, fileSystem @*/
+	/*@modifies *sig, *size, rpmGlobalMacroContext, fileSystem @*/
 {
     char * sigfile = alloca(1024);
     int pid, status;
     int inpipe[2];
     FILE * fpipe;
     struct stat st;
+    const char * cmd;
+    char *const *av;
+    int rc;
 
     (void) stpcpy( stpcpy(sigfile, file), ".sig");
 
+    addMacro(NULL, "__plaintext_filename", NULL, file, -1);
+    addMacro(NULL, "__signature_filename", NULL, sigfile, -1);
+
     inpipe[0] = inpipe[1] = 0;
     (void) pipe(inpipe);
-    
+
     if (!(pid = fork())) {
-	const char *gpg_path = rpmExpand("%{_gpg_path}", NULL);
-	const char *name = rpmExpand("%{_gpg_name}", NULL);
+	const char *gpg_path = rpmExpand("%{?_gpg_path}", NULL);
 
 	(void) close(STDIN_FILENO);
 	(void) dup2(inpipe[0], 3);
 	(void) close(inpipe[1]);
 
-	if (gpg_path && *gpg_path != '%')
+	if (gpg_path && *gpg_path != '\0')
 	    (void) dosetenv("GNUPGHOME", gpg_path, 1);
-	(void) execlp("gpg", "gpg",
-	       "--batch", "--no-verbose", "--no-armor", "--passphrase-fd", "3",
-	       "-u", name, "-sbo", sigfile, file,
-	       NULL);
-	rpmError(RPMERR_EXEC, _("Couldn't exec gpg\n"));
+
+	cmd = rpmExpand("%{?__gpg_sign_cmd}", NULL);
+	rc = poptParseArgvString(cmd, NULL, (const char ***)&av);
+	if (!rc)
+	    rc = execve(av[0], av+1, environ);
+
+	rpmError(RPMERR_EXEC, _("Could not exec %s: %s\n"), "gpg",
+			strerror(errno));
 	_exit(RPMERR_EXEC);
     }
 
+    delMacro(NULL, "__plaintext_filename");
+    delMacro(NULL, "__signature_filename");
+
     fpipe = fdopen(inpipe[1], "w");
     (void) close(inpipe[0]);
     if (fpipe) {
@@ -396,7 +418,7 @@ static int makeGPGSignature(const char * file, /*@out@*/ void ** sig,
     *size = st.st_size;
     rpmMessage(RPMMESS_DEBUG, _("GPG sig size: %d\n"), *size);
     *sig = xmalloc(*size);
-    
+
     {	FD_t fd;
 	int rc = 0;
 	fd = Fopen(sigfile, "r.fdio");
@@ -413,7 +435,7 @@ static int makeGPGSignature(const char * file, /*@out@*/ void ** sig,
     }
 
     rpmMessage(RPMMESS_DEBUG, _("Got %d bytes of GPG sig\n"), *size);
-    
+
     return 0;
 }
 
@@ -425,7 +447,7 @@ int rpmAddSignature(Header h, const char * file, int_32 sigTag,
     byte buf[16];
     void *sig;
     int ret = -1;
-    
+
     switch (sigTag) {
     case RPMSIGTAG_SIZE:
 	(void) stat(file, &st);
@@ -458,6 +480,7 @@ int rpmAddSignature(Header h, const char * file, int_32 sigTag,
 
 static rpmVerifySignatureReturn
 verifySizeSignature(const char * datafile, int_32 size, /*@out@*/ char * result)
+	/*@globals fileSystem @*/
 	/*@modifies *result, fileSystem @*/
 {
     struct stat st;
@@ -477,8 +500,9 @@ verifySizeSignature(const char * datafile, int_32 size, /*@out@*/ char * result)
 #define	X(_x)	(unsigned)((_x) & 0xff)
 
 static rpmVerifySignatureReturn
-verifyMD5Signature(const char * datafile, const byte * sig, 
+verifyMD5Signature(const char * datafile, const byte * sig,
 			      /*@out@*/ char * result, md5func fn)
+	/*@globals fileSystem @*/
 	/*@modifies *result, fileSystem @*/
 {
     byte md5sum[16];
@@ -515,7 +539,8 @@ verifyMD5Signature(const char * datafile, const byte * sig,
 static rpmVerifySignatureReturn
 verifyPGPSignature(const char * datafile, const void * sig, int count,
 		/*@out@*/ char * result)
-	/*@modifies *result, fileSystem @*/
+	/*@globals rpmGlobalMacroContext, fileSystem @*/
+	/*@modifies *result, rpmGlobalMacroContext, fileSystem @*/
 {
     int pid, status, outpipe[2];
 /*@only@*/ /*@null@*/ const char * sigfile = NULL;
@@ -524,12 +549,15 @@ verifyPGPSignature(const char * datafile, const void * sig, int count,
     int res = RPMSIG_OK;
     const char *path;
     pgpVersion pgpVer;
+    const char * cmd;
+    char *const *av;
+    int rc;
 
     /* What version do we have? */
     if ((path = rpmDetectPGPVersion(&pgpVer)) == NULL) {
 	errno = ENOENT;
-	rpmError(RPMERR_EXEC, 
-		 _("Could not run pgp.  Use --nopgp to skip PGP checks.\n"));
+	rpmError(RPMERR_EXEC, ("Could not exec %s: %s\n"), "pgp",
+			strerror(errno));
 	_exit(RPMERR_EXEC);
     }
 
@@ -563,51 +591,59 @@ verifyPGPSignature(const char * datafile, const void * sig, int count,
     if (sigfile == NULL)
 	return RPMSIG_BAD;
 
+    addMacro(NULL, "__plaintext_filename", NULL, datafile, -1);
+    addMacro(NULL, "__signature_filename", NULL, sigfile, -1);
+
     /* Now run PGP */
     outpipe[0] = outpipe[1] = 0;
     (void) pipe(outpipe);
 
     if (!(pid = fork())) {
-	const char *pgp_path = rpmExpand("%{_pgp_path}", NULL);
+	const char *pgp_path = rpmExpand("%{?_pgp_path}", NULL);
 
 	(void) close(outpipe[0]);
 	(void) close(STDOUT_FILENO);	/* XXX unnecessary */
 	(void) dup2(outpipe[1], STDOUT_FILENO);
 
-	if (pgp_path && *pgp_path != '%')
+	if (pgp_path && *pgp_path != '\0')
 	    (void) dosetenv("PGPPATH", pgp_path, 1);
 
 	switch (pgpVer) {
+	case PGP_2:
+	    cmd = rpmExpand("%{?__pgp_verify_cmd}", NULL);
+	    rc = poptParseArgvString(cmd, NULL, (const char ***)&av);
+	    if (!rc)
+		rc = execve(av[0], av+1, environ);
+	    break;
 	case PGP_5:
 	    /* Some output (in particular "This signature applies to */
 	    /* another message") is _always_ written to stderr; we   */
 	    /* want to catch that output, so dup stdout to stderr:   */
 	{   int save_stderr = dup(2);
 	    (void) dup2(1, 2);
-	    (void) execlp(path, "pgpv", "+batchmode=on", "+verbose=0",
-		   /* Write "Good signature..." to stdout: */
-		   "+OutputInformationFD=1",
-		   /* Write "WARNING: ... is not trusted to... to stdout: */
-		   "+OutputWarningFD=1",
-		   sigfile, "-o", datafile, NULL);
+
+	    cmd = rpmExpand("%{?__pgp5_verify_cmd}", NULL);
+	    rc = poptParseArgvString(cmd, NULL, (const char ***)&av);
+	    if (!rc)
+		rc = execve(av[0], av+1, environ);
+
 	    /* Restore stderr so we can print the error message below. */
 	    (void) dup2(save_stderr, 2);
 	    (void) close(save_stderr);
 	}   break;
-	case PGP_2:
-	    (void) execlp(path, "pgp", "+batchmode=on", "+verbose=0",
-		   sigfile, datafile, NULL);
-	    break;
 	case PGP_UNKNOWN:
 	case PGP_NOTDETECTED:
 	    break;
 	}
 
-	rpmError(RPMERR_EXEC, 
-		 _("Could not run pgp.  Use --nopgp to skip PGP checks.\n"));
+	rpmError(RPMERR_EXEC, _("Could not exec %s: %s\n"), "pgp",
+			strerror(errno));
 	_exit(RPMERR_EXEC);
     }
 
+    delMacro(NULL, "__plaintext_filename");
+    delMacro(NULL, "__signature_filename");
+
     (void) close(outpipe[1]);
     file = fdopen(outpipe[0], "r");
     result[0] = '\0';
@@ -637,21 +673,25 @@ verifyPGPSignature(const char * datafile, const void * sig, int count,
     if (!res && (!WIFEXITED(status) || WEXITSTATUS(status))) {
 	res = RPMSIG_BAD;
     }
-    
+
     return res;
 }
 
 static rpmVerifySignatureReturn
 verifyGPGSignature(const char * datafile, const void * sig, int count,
 		/*@out@*/ char * result)
-	/*@modifies *result, fileSystem @*/
+	/*@globals rpmGlobalMacroContext, fileSystem @*/
+	/*@modifies *result, rpmGlobalMacroContext, fileSystem @*/
 {
     int pid, status, outpipe[2];
 /*@only@*/ /*@null@*/ const char * sigfile = NULL;
     byte buf[BUFSIZ];
     FILE *file;
     int res = RPMSIG_OK;
-  
+    const char * cmd;
+    char *const *av;
+    int rc;
+
     /* Write out the signature */
 #ifdef	DYING
   { const char *tmppath = rpmGetPath("%{_tmppath}", NULL);
@@ -675,29 +715,36 @@ verifyGPGSignature(const char * datafile, const void * sig, int count,
     if (sigfile == NULL)
 	return RPMSIG_BAD;
 
+    addMacro(NULL, "__plaintext_filename", NULL, datafile, -1);
+    addMacro(NULL, "__signature_filename", NULL, sigfile, -1);
+
     /* Now run GPG */
     outpipe[0] = outpipe[1] = 0;
     (void) pipe(outpipe);
 
     if (!(pid = fork())) {
-	const char *gpg_path = rpmExpand("%{_gpg_path}", NULL);
+	const char *gpg_path = rpmExpand("%{?_gpg_path}", NULL);
 
 	(void) close(outpipe[0]);
 	/* gpg version 0.9 sends its output to stderr. */
 	(void) dup2(outpipe[1], STDERR_FILENO);
 
-	if (gpg_path && *gpg_path != '%')
+	if (gpg_path && *gpg_path != '\0')
 	    (void) dosetenv("GNUPGHOME", gpg_path, 1);
 
-	(void) execlp("gpg", "gpg",
-	       "--batch", "--no-verbose", 
-	       "--verify", sigfile, datafile,
-	       NULL);
-	rpmError(RPMERR_EXEC, 
-		 _("Could not run gpg.  Use --nogpg to skip GPG checks.\n"));
+	cmd = rpmExpand("%{?__gpg_verify_cmd}", NULL);
+	rc = poptParseArgvString(cmd, NULL, (const char ***)&av);
+	if (!rc)
+	    rc = execve(av[0], av+1, environ);
+
+	rpmError(RPMERR_EXEC, _("Could not exec %s: %s\n"), "gpg",
+			strerror(errno));
 	_exit(RPMERR_EXEC);
     }
 
+    delMacro(NULL, "__plaintext_filename");
+    delMacro(NULL, "__signature_filename");
+
     (void) close(outpipe[1]);
     file = fdopen(outpipe[0], "r");
     result[0] = '\0';
@@ -710,23 +757,27 @@ verifyGPGSignature(const char * datafile, const void * sig, int count,
 	}
 	(void) fclose(file);
     }
-  
+
     (void) waitpid(pid, &status, 0);
     if (sigfile) (void) unlink(sigfile);
     sigfile = _free(sigfile);
     if (!res && (!WIFEXITED(status) || WEXITSTATUS(status))) {
 	res = RPMSIG_BAD;
     }
-    
+
     return res;
 }
 
 static int checkPassPhrase(const char * passPhrase, const int sigTag)
-	/*@modifies fileSystem @*/
+	/*@globals rpmGlobalMacroContext, fileSystem @*/
+	/*@modifies rpmGlobalMacroContext, fileSystem @*/
 {
     int passPhrasePipe[2];
     int pid, status;
     int fd;
+    const char * cmd;
+    char *const *av;
+    int rc;
 
     passPhrasePipe[0] = passPhrasePipe[1] = 0;
     (void) pipe(passPhrasePipe);
@@ -749,44 +800,51 @@ static int checkPassPhrase(const char * passPhrase, const int sigTag)
 
 	switch (sigTag) {
 	case RPMSIGTAG_GPG:
-	{   const char *gpg_path = rpmExpand("%{_gpg_path}", NULL);
-	    const char *name = rpmExpand("%{_gpg_name}", NULL);
-	    if (gpg_path && *gpg_path != '%')
+	{   const char *gpg_path = rpmExpand("%{?_gpg_path}", NULL);
+
+	    if (gpg_path && *gpg_path != '\0')
 		(void) dosetenv("GNUPGHOME", gpg_path, 1);
-	    (void) execlp("gpg", "gpg",
-	           "--batch", "--no-verbose", "--passphrase-fd", "3",
-	           "-u", name, "-so", "-",
-	           NULL);
-	    rpmError(RPMERR_EXEC, _("Couldn't exec gpg\n"));
+
+	    cmd = rpmExpand("%{?__gpg_check_password_cmd}", NULL);
+	    rc = poptParseArgvString(cmd, NULL, (const char ***)&av);
+	    if (!rc)
+		rc = execve(av[0], av+1, environ);
+
+	    rpmError(RPMERR_EXEC, _("Could not exec %s: %s\n"), "gpg",
+			strerror(errno));
 	    _exit(RPMERR_EXEC);
 	}   /*@notreached@*/ break;
 	case RPMSIGTAG_PGP5:	/* XXX legacy */
 	case RPMSIGTAG_PGP:
-	{   const char *pgp_path = rpmExpand("%{_pgp_path}", NULL);
-	    const char *name = rpmExpand("+myname=\"%{_pgp_name}\"", NULL);
+	{   const char *pgp_path = rpmExpand("%{?_pgp_path}", NULL);
 	    const char *path;
 	    pgpVersion pgpVer;
 
 	    (void) dosetenv("PGPPASSFD", "3", 1);
-	    if (pgp_path && *pgp_path != '%')
+	    if (pgp_path && *pgp_path != '\0')
 		(void) dosetenv("PGPPATH", pgp_path, 1);
 
 	    if ((path = rpmDetectPGPVersion(&pgpVer)) != NULL) {
 		switch(pgpVer) {
 		case PGP_2:
-		    (void) execlp(path, "pgp", "+batchmode=on", "+verbose=0",
-			name, "-sf", NULL);
+		    cmd = rpmExpand("%{?__pgp_check_password_cmd}", NULL);
+		    rc = poptParseArgvString(cmd, NULL, (const char ***)&av);
+		    if (!rc)
+			rc = execve(av[0], av+1, environ);
 		    break;
 		case PGP_5:	/* XXX legacy */
-		    (void) execlp(path,"pgps", "+batchmode=on", "+verbose=0",
-			name, "-f", NULL);
+		    cmd = rpmExpand("%{?__pgp5_check_password_cmd}", NULL);
+		    rc = poptParseArgvString(cmd, NULL, (const char ***)&av);
+		    if (!rc)
+			rc = execve(av[0], av+1, environ);
 		    break;
 		case PGP_UNKNOWN:
 		case PGP_NOTDETECTED:
 		    break;
 		}
 	    }
-	    rpmError(RPMERR_EXEC, _("Couldn't exec pgp\n"));
+	    rpmError(RPMERR_EXEC, _("Could not exec %s: %s\n"), "pgp",
+			strerror(errno));
 	    _exit(RPMERR_EXEC);
 	}   /*@notreached@*/ break;
 	default: /* This case should have been screened out long ago. */
@@ -817,8 +875,8 @@ char * rpmGetPassPhrase(const char * prompt, const int sigTag)
 
     switch (sigTag) {
     case RPMSIGTAG_GPG:
-      { const char *name = rpmExpand("%{_gpg_name}", NULL);
-	aok = (name && *name != '%');
+      { const char *name = rpmExpand("%{?_gpg_name}", NULL);
+	aok = (name && *name != '\0');
 	name = _free(name);
       }
 	if (!aok) {
@@ -828,9 +886,9 @@ char * rpmGetPassPhrase(const char * prompt, const int sigTag)
 	}
 	break;
     case RPMSIGTAG_PGP5: 	/* XXX legacy */
-    case RPMSIGTAG_PGP: 
-      { const char *name = rpmExpand("%{_pgp_name}", NULL);
-	aok = (name && *name != '%');
+    case RPMSIGTAG_PGP:
+      { const char *name = rpmExpand("%{?_pgp_name}", NULL);
+	aok = (name && *name != '\0');
 	name = _free(name);
       }
 	if (!aok) {
@@ -860,28 +918,31 @@ rpmVerifySignatureReturn
 rpmVerifySignature(const char * file, int_32 sigTag, const void * sig,
 		int count, char * result)
 {
+     rpmVerifySignatureReturn res;
+
     switch (sigTag) {
     case RPMSIGTAG_SIZE:
-	return verifySizeSignature(file, *(int_32 *)sig, result);
-	/*@notreached@*/ break;
+	res = verifySizeSignature(file, *(int_32 *)sig, result);
+	break;
     case RPMSIGTAG_MD5:
-	return verifyMD5Signature(file, sig, result, mdbinfile);
-	/*@notreached@*/ break;
-    case RPMSIGTAG_LEMD5_1:
-    case RPMSIGTAG_LEMD5_2:
-	return verifyMD5Signature(file, sig, result, mdbinfileBroken);
-	/*@notreached@*/ break;
+	res = verifyMD5Signature(file, sig, result, mdbinfile);
+	break;
     case RPMSIGTAG_PGP5:	/* XXX legacy */
     case RPMSIGTAG_PGP:
-	return verifyPGPSignature(file, sig, count, result);
-	/*@notreached@*/ break;
+	res = verifyPGPSignature(file, sig, count, result);
+	break;
     case RPMSIGTAG_GPG:
-	return verifyGPGSignature(file, sig, count, result);
-	/*@notreached@*/ break;
+	res = verifyGPGSignature(file, sig, count, result);
+	break;
+    case RPMSIGTAG_LEMD5_1:
+    case RPMSIGTAG_LEMD5_2:
+	sprintf(result, _("Broken MD5 digest: UNSUPPORTED\n"));
+	res = RPMSIG_UNKNOWN;
+	break;
     default:
 	sprintf(result, "Do not know how to verify sig type %d\n", sigTag);
-	return RPMSIG_UNKNOWN;
+	res = RPMSIG_UNKNOWN;
+	break;
     }
-    /*@notreached@*/
-    return RPMSIG_OK;
+    return res;
 }
