@@ -16,6 +16,25 @@ struct StringBufRec {
     int free;
 };
 
+/**
+ * Locale insensitive isspace(3).
+ */
+/*@unused@*/ static inline int xisspace(int c) {
+    return (c == ' ' || c == '\t' || c == '\n' || c == '\r' || c == '\f' || c == '\v');
+}
+
+/**
+ * Wrapper to free(3), hides const compilation noise, permit NULL, return NULL.
+ * @param p		memory to free
+ * @return		NULL always
+ */
+/*@unused@*/ static inline /*@null@*/ void *
+_free(/*@only@*/ /*@null@*/ const void * p) /*@modifies *p @*/
+{
+    if (p != NULL)	free((void *)p);
+    return NULL;
+}
+
 StringBuf newStringBuf(void)
 {
     StringBuf sb = xmalloc(sizeof(struct StringBufRec));
@@ -28,14 +47,13 @@ StringBuf newStringBuf(void)
     return sb;
 }
 
-void freeStringBuf(StringBuf sb)
+StringBuf freeStringBuf(StringBuf sb)
 {
-    if (! sb) {
-	return;
+    if (sb) {
+	sb->buf = _free(sb->buf);
+	sb = _free(sb);
     }
-    
-    free(sb->buf);
-    free(sb);
+    return sb;
 }
 
 void truncStringBuf(StringBuf sb)
@@ -48,7 +66,7 @@ void truncStringBuf(StringBuf sb)
 void stripTrailingBlanksStringBuf(StringBuf sb)
 {
     while (sb->free != sb->allocated) {
-	if (! isspace(*(sb->tail - 1))) {
+	if (! xisspace(*(sb->tail - 1))) {
 	    break;
 	}
 	sb->free++;
@@ -57,7 +75,7 @@ void stripTrailingBlanksStringBuf(StringBuf sb)
     sb->tail[0] = '\0';
 }
 
-char *getStringBuf(StringBuf sb)
+char * getStringBuf(StringBuf sb)
 {
     return sb->buf;
 }
@@ -75,7 +93,9 @@ void appendStringBufAux(StringBuf sb, const char *s, int nl)
 	sb->tail = sb->buf + (sb->allocated - sb->free);
     }
     
+    /*@-mayaliasunique@*/
     strcpy(sb->tail, s);
+    /*@=mayaliasunique@*/
     sb->tail += l;
     sb->free -= l;
     if (nl) {
