@@ -40,9 +40,11 @@ const char *const tagName(int tag)
     for (i = 0; i < rpmTagTableSize; i++) {
 	if (tag != rpmTagTable[i].val)
 	    continue;
-	strcpy(nameBuf, rpmTagTable[i].name + 7);
-	for (s = nameBuf+1; *s; s++)
-	    *s = tolower(*s);
+	nameBuf[0] = nameBuf[1] = '\0';
+	if (rpmTagTable[i].name != NULL)	/* XXX programmer error. */
+	    strcpy(nameBuf, rpmTagTable[i].name + 7);
+	for (s = nameBuf+1; *s != '\0'; s++)
+	    *s = xtolower(*s);
 	break;
     }
     return nameBuf;
@@ -50,7 +52,7 @@ const char *const tagName(int tag)
 
 int tagValue(const char * tagstr)
 {
-    const struct headerTagTableEntry *t;
+    const struct headerTagTableEntry_s *t;
 
     if (!strcmp(tagstr, "Packages"))
 	return RPMDBI_PACKAGES;
@@ -64,7 +66,7 @@ int tagValue(const char * tagstr)
 	return RPMDBI_AVAILABLE;
 
     for (t = rpmTagTable; t->name != NULL; t++) {
-	if (!strcasecmp(t->name + 7, tagstr))
+	if (!xstrcasecmp(t->name + 7, tagstr))
 	    return t->val;
     }
     return -1;
