@@ -1,7 +1,7 @@
 #include "system.h"
 #include "rpmlib.h"
 
-const struct headerTagTableEntry rpmTagTable[] = {
+const struct headerTagTableEntry_s rpmTagTable[] = {
 	{ "RPMTAG_HEADERIMAGE", HEADER_IMAGE, },
 	{ "RPMTAG_HEADERSIGNATURES", HEADER_SIGNATURES, },
 	{ "RPMTAG_HEADERIMMUTABLE", HEADER_IMMUTABLE, },
@@ -11,6 +11,7 @@ const struct headerTagTableEntry rpmTagTable[] = {
 	{ "RPMTAG_SIGPGP", RPMTAG_SIG_BASE+3, },
 	{ "RPMTAG_SIGMD5", RPMTAG_SIG_BASE+5, },
 	{ "RPMTAG_SIGGPG", RPMTAG_SIG_BASE+6, },
+	{ "RPMTAG_SHA1HEADER", RPMTAG_SIG_BASE+9, },
 	{ "RPMTAG_NAME", 1000, },
 	{ "RPMTAG_VERSION", 1001, },
 	{ "RPMTAG_RELEASE", 1002, },
@@ -48,7 +49,6 @@ const struct headerTagTableEntry rpmTagTable[] = {
 	{ "RPMTAG_FILEMD5S", 1035, },
 	{ "RPMTAG_FILELINKTOS", 1036, },
 	{ "RPMTAG_FILEFLAGS", 1037, },
-	{ "RPMTAG_ROOT", 1038, },
 	{ "RPMTAG_FILEUSERNAME", 1039, },
 	{ "RPMTAG_FILEGROUPNAME", 1040, },
 	{ "RPMTAG_ICON", 1043, },
@@ -63,7 +63,6 @@ const struct headerTagTableEntry rpmTagTable[] = {
 	{ "RPMTAG_CONFLICTFLAGS", 1053, },
 	{ "RPMTAG_CONFLICTNAME", 1054, },
 	{ "RPMTAG_CONFLICTVERSION", 1055, },
-	{ "RPMTAG_BUILDROOT", 1057, },
 	{ "RPMTAG_EXCLUDEARCH", 1059, },
 	{ "RPMTAG_EXCLUDEOS", 1060, },
 	{ "RPMTAG_EXCLUSIVEARCH", 1061, },
@@ -93,8 +92,6 @@ const struct headerTagTableEntry rpmTagTable[] = {
 	{ "RPMTAG_FILELANGS", 1097, },
 	{ "RPMTAG_PREFIXES", 1098, },
 	{ "RPMTAG_INSTPREFIXES", 1099, },
-	{ "RPMTAG_OLDORIGFILENAMES", 1107, },
-	{ "RPMTAG_BUILDMACROS", 1111, },
 	{ "RPMTAG_PROVIDEFLAGS", 1112, },
 	{ "RPMTAG_PROVIDEVERSION", 1113, },
 	{ "RPMTAG_OBSOLETEFLAGS", 1114, },
@@ -110,7 +107,9 @@ const struct headerTagTableEntry rpmTagTable[] = {
 	{ "RPMTAG_MULTILIBS", 1127, },
 	{ "RPMTAG_INSTALLTID", 1128, },
 	{ "RPMTAG_REMOVETID", 1129, },
+	{ "RPMTAG_RHNPLATFORM", 1131, },
+	{ "RPMTAG_PLATFORM", 1132, },
 	{ NULL, 0 }
 };
 
-const int rpmTagTableSize = sizeof(rpmTagTable) / sizeof(struct headerTagTableEntry) - 1;
+const int rpmTagTableSize = sizeof(rpmTagTable) / sizeof(rpmTagTable[0]) - 1;
