@@ -4,19 +4,20 @@
 
 #include "system.h"
 
-#include <rpmlib.h>
 #include <rpmmacro.h>	/* XXX for rpmExpand */
 
 #include "psm.h"
+
+#include "rpmdb.h"
 #include "fprint.h"
 #include "rpmhash.h"
-#include "md5.h"
 #include "misc.h" /* XXX stripTrailingChar, splitString, currentDirectory */
-#include "rpmdb.h"
 
+#ifdef	DYING
 /*@-redecl -exportheadervar@*/
 extern const char * chroot_prefix;
 /*@=redecl =exportheadervar@*/
+#endif
 
 /* XXX FIXME: merge with existing (broken?) tests in system.h */
 /* portability fiddles */
@@ -785,7 +786,7 @@ static fileAction decideFileFate(const char * dirName,
 			const char * baseName, short dbMode,
 			const char * dbMd5, const char * dbLink, short newMode,
 			const char * newMd5, const char * newLink, int newFlags,
-			int brokenMd5, rpmtransFlags transFlags)
+			rpmtransFlags transFlags)
 	/*@*/
 {
     char buffer[1024];
@@ -834,11 +835,7 @@ static fileAction decideFileFate(const char * dirName,
     }
 
     if (dbWhat == REG) {
-	if (brokenMd5)
-	    rc = mdfileBroken(filespec, buffer);
-	else
-	    rc = mdfile(filespec, buffer);
-
+	rc = domd5(filespec, buffer, 0);
 	if (rc) {
 	    /* assume the file has been removed, don't freak */
 	    return FA_CREATE;
@@ -974,7 +971,6 @@ static int handleInstInstalledFiles(TFI_t fi, /*@null@*/ rpmdb db,
 			fi->fmd5s[fileNum],
 			fi->flinks[fileNum],
 			fi->fflags[fileNum],
-			!headerIsEntry(h, RPMTAG_RPMVERSION),
 			transFlags);
 	}
 
@@ -1252,7 +1248,8 @@ static int ensureOlder(struct availablePackage * alp, Header old,
 }
 
 static void skipFiles(const rpmTransactionSet ts, TFI_t fi)
-	/*@modifies fi @*/
+	/*@globals rpmGlobalMacroContext @*/
+	/*@modifies fi, rpmGlobalMacroContext @*/
 {
     int noDocs = (ts->transFlags & RPMTRANS_FLAG_NODOCS);
     char ** netsharedPaths = NULL;
@@ -1524,9 +1521,6 @@ int rpmRunTransactions(	rpmTransactionSet ts,
     int i, j;
     int ourrc = 0;
     struct availablePackage * alp;
-#ifdef	DYING
-    Header * hdrs;
-#endif
     int totalFileCount = 0;
     hashTable ht;
     TFI_t fi;
@@ -1625,10 +1619,6 @@ int rpmRunTransactions(	rpmTransactionSet ts,
 	if (dip) ts->di[i].bsize = 0;
     }
 
-#ifdef	DYING
-    hdrs = alloca(sizeof(*hdrs) * ts->addedPackages.size);
-#endif
-
     /* ===============================================
      * For packages being installed:
      * - verify package arch/os.
@@ -1720,22 +1710,12 @@ int rpmRunTransactions(	rpmTransactionSet ts,
 	    fi->ap = tsGetAlp(tsi);
 	    fi->record = 0;
 	    loadFi(fi->ap->h, fi);
-	    if (fi->fc == 0) {
-#ifdef	DYING
-		hdrs[i] = headerLink(fi->h);
-#endif
+	    if (fi->fc == 0)
 		continue;
-	    }
 
-#ifdef	DYING
-	    /* Allocate file actions (and initialize to FA_UNKNOWN) */
-	    fi->actions = xcalloc(fi->fc, sizeof(*fi->actions));
-	    hdrs[i] = relocateFileList(ts, fi, fi->ap, fi->h, fi->actions);
-#else
 	    {   Header foo = relocateFileList(ts, fi, fi->ap, fi->h, fi->actions);
 		foo = headerFree(foo);
 	    }
-#endif
 
 	    /* Skip netshared paths, not our i18n files, and excluded docs */
 	    skipFiles(ts, fi);
@@ -1765,11 +1745,6 @@ int rpmRunTransactions(	rpmTransactionSet ts,
     }
     tsi = tsFreeIterator(tsi);
 
-#ifdef	DYING
-    /* Open all database indices before installing. */
-    (void) rpmdbOpenAll(ts->rpmdb);
-#endif
-
     if (!ts->chrootDone) {
 	(void) chdir("/");
 	/*@-unrecog -superuser @*/
@@ -1777,9 +1752,11 @@ int rpmRunTransactions(	rpmTransactionSet ts,
 	/*@=unrecog =superuser @*/
 	ts->chrootDone = 1;
 	if (ts->rpmdb) ts->rpmdb->db_chrootDone = 1;
+#ifdef	DYING
 	/*@-onlytrans@*/
 	chroot_prefix = ts->rootDir;
 	/*@=onlytrans@*/
+#endif
     }
 
     ht = htCreate(totalFileCount * 2, 0, 0, fpHashFunction, fpEqual);
@@ -1945,7 +1922,9 @@ int rpmRunTransactions(	rpmTransactionSet ts,
 	/*@=unrecog =superuser @*/
 	ts->chrootDone = 0;
 	if (ts->rpmdb) ts->rpmdb->db_chrootDone = 0;
+#ifdef	DYING
 	chroot_prefix = NULL;
+#endif
 	(void) chdir(ts->currDir);
     }
 
@@ -1978,16 +1957,6 @@ int rpmRunTransactions(	rpmTransactionSet ts,
     {
 	*newProbs = ts->probs;
 
-#ifdef	DYING
-	for (alp = ts->addedPackages.list, fi = ts->flList;
-		(alp - ts->addedPackages.list) < ts->addedPackages.size;
-		alp++, fi++)
-	{
-	    hdrs[alp - ts->addedPackages.list] =
-		headerFree(hdrs[alp - ts->addedPackages.list]);
-	}
-#endif
-
 	ts->flList = freeFl(ts, ts->flList);
 	ts->flEntries = 0;
 	/*@-nullstate@*/
@@ -2040,11 +2009,8 @@ assert(alp == fi->ap);
 		if (alp->fd) {
 		    rpmRC rpmrc;
 
-#ifdef	DYING
-		    hdrs[i] = headerFree(hdrs[i]);
-#else
 		    h = headerFree(h);
-#endif
+
 		    /*@-mustmod@*/	/* LCL: segfault */
 		    rpmrc = rpmReadPackageHeader(alp->fd, &h, NULL, NULL, NULL);
 		    /*@=mustmod@*/
@@ -2054,15 +2020,10 @@ assert(alp == fi->ap);
 			alp->fd = NULL;
 			ourrc++;
 		    } else {
-#ifdef	DYING
-			hdrs[i] = relocateFileList(ts, fi, alp, h, NULL);
-			h = headerFree(h);
-#else
 			Header foo = relocateFileList(ts, fi, alp, h, NULL);
 			h = headerFree(h);
 			h = headerLink(foo);
 			foo = headerFree(foo);
-#endif
 		    }
 		    if (alp->fd) gotfd = 1;
 		}
@@ -2075,11 +2036,7 @@ assert(alp == fi->ap);
 		    hsave = headerLink(fi->h);
 		    fi->h = headerFree(fi->h);
 		}
-#ifdef	DYING
-		fi->h = headerLink(hdrs[i]);
-#else
 		fi->h = headerLink(h);
-#endif
 		if (alp->multiLib)
 		    ts->transFlags |= RPMTRANS_FLAG_MULTILIB;
 
@@ -2098,11 +2055,7 @@ assert(alp == fi->ap);
 		lastFailed = i;
 	    }
 
-#ifdef	DYING
-	    hdrs[i] = headerFree(hdrs[i]);
-#else
 	    h = headerFree(h);
-#endif
 
 	    if (gotfd) {
 		(void)ts->notify(fi->h, RPMCALLBACK_INST_CLOSE_FILE, 0, 0,
