@@ -5,10 +5,10 @@
 
 #include "system.h"
 
+#include "psm.h"
 #include <rpmcli.h>
 
-#include "psm.h"
-#include "md5.h"
+#include "ugid.h"
 #include "misc.h"	/* XXX for uidToUname() and gnameToGid() */
 #include "debug.h"
 
@@ -17,14 +17,6 @@
 /*@access FD_t*/	/* XXX compared with NULL */
 /*@access rpmdb*/	/* XXX compared with NULL */
 
-static int _ie = 0x44332211;
-static union _vendian {
-/*@unused@*/ int i;
-    char b[4];
-} *_endian = (union _vendian *)&_ie;
-#define	IS_BIG_ENDIAN()		(_endian->b[0] == '\x44')
-#define	IS_LITTLE_ENDIAN()	(_endian->b[0] == '\x11')
-
 #define S_ISDEV(m) (S_ISBLK((m)) || S_ISCHR((m)))
 
 int rpmVerifyFile(const char * root, Header h, int filenum,
@@ -42,29 +34,8 @@ int rpmVerifyFile(const char * root, Header h, int filenum,
     int count;
     int rc;
     struct stat sb;
-    int_32 useBrokenMd5;
 
-  if (IS_BIG_ENDIAN()) {	/* XXX was ifdef WORDS_BIGENDIAN */
-    int_32 * brokenPtr;
-    if (!hge(h, RPMTAG_BROKENMD5, NULL, (void **) &brokenPtr, NULL)) {
-	HAE_t hae = (HAE_t)headerAddEntry;
-	const char * rpmVersion;
-
-	if (hge(h, RPMTAG_RPMVERSION, NULL, (void **) &rpmVersion, NULL)) {
-	    useBrokenMd5 = ((rpmvercmp(rpmVersion, "2.3.3") >= 0) &&
-			    (rpmvercmp(rpmVersion, "2.3.8") <= 0));
-	} else {
-	    useBrokenMd5 = 1;
-	}
-	(void) hae(h, RPMTAG_BROKENMD5, RPM_INT32_TYPE, &useBrokenMd5, 1);
-    } else {
-	useBrokenMd5 = *brokenPtr;
-    }
-  } else {
-    useBrokenMd5 = 0;
-  }
-
-    (void) hge(h, RPMTAG_FILEMODES, NULL, (void **) &modeList, &count);
+    rc = hge(h, RPMTAG_FILEMODES, NULL, (void **) &modeList, &count);
     if (hge(h, RPMTAG_FILEFLAGS, NULL, (void **) &fileFlags, NULL))
 	fileAttrs = fileFlags[filenum];
 
@@ -175,11 +146,7 @@ int rpmVerifyFile(const char * root, Header h, int filenum,
 	if (!hge(h, RPMTAG_FILEMD5S, &mdt, (void **) &md5List, NULL))
 	    *result |= RPMVERIFY_MD5;
 	else {
-	    if (useBrokenMd5)
-		rc = mdfileBroken(filespec, md5sum);
-	    else
-		rc = mdfile(filespec, md5sum);
-
+	    rc = domd5(filespec, md5sum, 1);
 	    if (rc)
 		*result |= (RPMVERIFY_READFAIL|RPMVERIFY_MD5);
 	    else if (strcmp(md5sum, md5List[filenum]))
@@ -284,7 +251,7 @@ int rpmVerifyFile(const char * root, Header h, int filenum,
 	gid_t gid;
 
 	if (hge(h, RPMTAG_FILEGROUPNAME, &gnt, (void **) &gnameList, NULL)) {
-	    rc =  gnameToGid(gnameList[filenum], &gid);
+	    rc = gnameToGid(gnameList[filenum], &gid);
 	    if (rc || (gid != sb.st_gid))
 		*result |= RPMVERIFY_GROUP;
 	    gnameList = hfd(gnameList, gnt);
@@ -344,7 +311,6 @@ int rpmVerifyDigest(Header h)
     int_32 uhc;
     const char * hdigest;
     rpmTagType hdt;
-    int flags = RPMDIGEST_SHA1;
     int ec = 0;		/* assume no problems */
 
     /* Retrieve header digest. */
@@ -352,10 +318,9 @@ int rpmVerifyDigest(Header h)
     &&	!hge(h, RPMTAG_SHA1RHN, &hdt, (void **) &hdigest, NULL))
     {
 	if (hge(h, RPMTAG_BADSHA1HEADER, &hdt, (void **) &hdigest, NULL))
-	    flags |= (RPMDIGEST_REVERSE|RPMDIGEST_BCSWAP);
-	else
 	    return 0;
     }
+
     /* Regenerate original header. */
     if (!hge(h, RPMTAG_HEADERIMMUTABLE, &uht, &uh, &uhc))
 	return 0;
@@ -364,12 +329,12 @@ int rpmVerifyDigest(Header h)
 	return 0;
 
     /* Compute header digest. */
-    {	DIGEST_CTX ctx = rpmDigestInit(flags);
+    {	DIGEST_CTX ctx = rpmDigestInit(PGPHASHALGO_SHA1, RPMDIGEST_NONE);
 	const char * digest;
 	size_t digestlen;
 
-	rpmDigestUpdate(ctx, uh, uhc);
-	rpmDigestFinal(ctx, (void **)&digest, &digestlen, 1);
+	(void) rpmDigestUpdate(ctx, uh, uhc);
+	(void) rpmDigestFinal(ctx, (void **)&digest, &digestlen, 1);
 
 	/* XXX can't happen: report NULL malloc return as a digest failure. */
 	ec = (digest == NULL || strcmp(hdigest, digest)) ? 1 : 0;
@@ -426,11 +391,13 @@ static int verifyHeader(QVA_t qva, Header h)
 
 	rc = rpmVerifyFile(prefix, h, i, &verifyResult, omitMask);
 	if (rc) {
+	    /*@-internalglobs@*/ /* FIX: shrug */
 	    if (!(fileAttrs & RPMFILE_MISSINGOK) || rpmIsVerbose()) {
 		sprintf(te, _("missing    %s"), fileNames[i]);
 		te += strlen(te);
 		ec = rc;
 	    }
+	    /*@=internalglobs@*/
 	} else if (verifyResult) {
 	    const char * size, * md5, * link, * mtime, * mode;
 	    const char * group, * user, * rdev;
@@ -507,6 +474,7 @@ static int verifyDependencies(rpmdb rpmdb, Header h)
     (void) rpmdepCheck(ts, &conflicts, &numConflicts);
     ts = rpmtransFree(ts);
 
+    /*@-branchstate@*/
     if (numConflicts) {
 	const char *n, *v, *r;
 	char * t, * te;
@@ -545,6 +513,7 @@ static int verifyDependencies(rpmdb rpmdb, Header h)
 	}
 	rc = 1;
     }
+    /*@=branchstate@*/
     return rc;
 }
 
