@@ -15,9 +15,9 @@
 #include <time.h>
 
 #include "Python.h"
+#include "rpmio_internal.h"
 #include "rpmcli.h"	/* XXX for rpmCheckSig */
 #include "misc.h"
-#include "rpmio_internal.h"
 #include "header_internal.h"
 #include "upgrade.h"
 
@@ -103,7 +103,9 @@ static PyObject * pyrpmError;
  * 	print header['name']
  * \endcode
  * This method of access is a bit slower because the name must be
- * translated into the tag number dynamically.
+ * translated into the tag number dynamically. You also must make sure
+ * the strings in header lookups don't get translated, or the lookups
+ * will fail.
  */
 
 /** \ingroup python
@@ -485,7 +487,7 @@ static PyObject * rhnUnload(hdrObject * s, PyObject * args) {
 
 	headerGetEntry(h, RPMTAG_HEADERIMMUTABLE, &uht, (void **)&uh, &uhc);
 
-	ctx = rpmDigestInit(RPMDIGEST_SHA1);
+	ctx = rpmDigestInit(PGPHASHALGO_SHA1, RPMDIGEST_NONE);
         rpmDigestUpdate(ctx, uh, uhc);
         rpmDigestFinal(ctx, (void **)&digest, &digestlen, 1);
 
@@ -508,12 +510,39 @@ static PyObject * rhnUnload(hdrObject * s, PyObject * args) {
 /** \ingroup python
  */
 static PyObject * hdrFullFilelist(hdrObject * s, PyObject * args) {
+    if (!PyArg_ParseTuple(args, ""))
+        return NULL;
+
     mungeFilelist (s->h);
 
     Py_INCREF(Py_None);
     return Py_None;
 }
 
+/** \ingroup python
+ */
+static PyObject * hdrSprintf(hdrObject * s, PyObject * args) {
+    char * fmt;
+    char * r;
+    errmsg_t err;
+    PyObject * result;
+
+    if (!PyArg_ParseTuple(args, "s", &fmt))
+        return NULL;
+
+    r = headerSprintf(s->h, fmt, rpmTagTable, rpmHeaderFormats, &err);
+    if (!r) {
+	PyErr_SetString(pyrpmError, err);
+	return NULL;
+    }
+
+    result = Py_BuildValue("s", r);
+    free(r);
+
+    return result;
+}
+
+
 /** \ingroup python
  */
 static struct PyMethodDef hdrMethods[] = {
@@ -524,6 +553,7 @@ static struct PyMethodDef hdrMethods[] = {
 	{"compressFilelist",	(PyCFunction) hdrCompressFilelist,	1 },
 	{"fullFilelist",	(PyCFunction) hdrFullFilelist,	1 },
 	{"rhnUnload",	(PyCFunction) rhnUnload, METH_VARARGS },
+	{"sprintf",	(PyCFunction) hdrSprintf, METH_VARARGS },
 	{NULL,		NULL}		/* sentinel */
 };
 
@@ -2318,6 +2348,9 @@ static PyObject * checkSig (PyObject * self, PyObject * args) {
  */
 static PyObject * getTsHeader (PyObject * self, PyObject * args) {
     hdrObject * h;
+
+    if (!PyArg_ParseTuple(args, ""))
+	return NULL;
     
     if (transactionSetHeader) {
 	h = (hdrObject *) PyObject_NEW(PyObject, &hdrType);
@@ -2332,6 +2365,18 @@ static PyObject * getTsHeader (PyObject * self, PyObject * args) {
     return (PyObject *) Py_None;
 }
 
+static PyObject * setVerbosity (PyObject * self, PyObject * args) {
+    int level;
+
+    if (!PyArg_ParseTuple(args, "i", &level))
+	return NULL;
+
+    rpmSetVerbosity(level);
+
+    Py_INCREF(Py_None);
+    return (PyObject *) Py_None;
+}
+
 /**
  */
 typedef struct FDlist_t FDlist;
@@ -2458,6 +2503,7 @@ static PyMethodDef rpmModuleMethods[] = {
     { "checksig", (PyCFunction) checkSig, METH_VARARGS, NULL },
     { "getTransactionCallbackHeader", (PyCFunction) getTsHeader, METH_VARARGS, NULL },
 /*      { "Fopen", (PyCFunction) doFopen, METH_VARARGS, NULL }, */
+    { "setVerbosity", (PyCFunction) setVerbosity, METH_VARARGS, NULL },
     { NULL }
 } ;
 
@@ -2604,6 +2650,16 @@ void initrpm(void) {
     REGISTER_ENUM(CHECKSIG_PGP);
     REGISTER_ENUM(CHECKSIG_GPG);
     REGISTER_ENUM(CHECKSIG_MD5);
+
+    REGISTER_ENUM(RPMLOG_EMERG);
+    REGISTER_ENUM(RPMLOG_ALERT);
+    REGISTER_ENUM(RPMLOG_CRIT);
+    REGISTER_ENUM(RPMLOG_ERR);
+    REGISTER_ENUM(RPMLOG_WARNING);
+    REGISTER_ENUM(RPMLOG_NOTICE);
+    REGISTER_ENUM(RPMLOG_INFO);
+    REGISTER_ENUM(RPMLOG_DEBUG);
+
 }
 
 /*@}*/
