@@ -1,7 +1,9 @@
+/*@-type@*/ /* FIX: annotate db3 methods */
 /** \ingroup db3
  * \file rpmdb/db3.c
  */
 
+/*@unchecked@*/
 static int _debug = 1;	/* XXX if < 0 debugging, > 0 unusual error returns */
 
 #include "system.h"
@@ -10,6 +12,17 @@ static int _debug = 1;	/* XXX if < 0 debugging, > 0 unusual error returns */
 #include <sys/ipc.h>
 #endif
 
+#if defined(__LCLINT__)
+/*@-redef@*/ /* FIX: rpmio/rpmio.c also declares */
+typedef	unsigned int u_int32_t;
+typedef	unsigned short u_int16_t;
+typedef	unsigned char u_int8_t;
+/*@-incondefs@*/	/* LCLint 3.0.0.15 */
+typedef	int int32_t;
+/*@=incondefs@*/
+/*@=redef@*/
+#endif
+
 #include <db3/db.h>
 
 #include <rpmlib.h>
@@ -72,7 +85,9 @@ struct dbiBStats_s {
 };
 /*@=fielduse@*/
 
+/*@-globuse -mustmod @*/	/* FIX: rpmError not annotated yet. */
 static int cvtdberr(dbiIndex dbi, const char * msg, int error, int printit)
+	/*@globals fileSystem @*/
 	/*@modifies fileSystem @*/
 {
     int rc = 0;
@@ -80,30 +95,32 @@ static int cvtdberr(dbiIndex dbi, const char * msg, int error, int printit)
     rc = error;
 
     if (printit && rc) {
+	/*@-moduncon@*/ /* FIX: annotate db3 methods */
 	if (msg)
 	    rpmError(RPMERR_DBERR, _("db%d error(%d) from %s: %s\n"),
 		dbi->dbi_api, rc, msg, db_strerror(error));
 	else
 	    rpmError(RPMERR_DBERR, _("db%d error(%d): %s\n"),
 		dbi->dbi_api, rc, db_strerror(error));
+	/*@=moduncon@*/
     }
 
     return rc;
 }
+/*@=globuse =mustmod @*/
 
 static int db_fini(dbiIndex dbi, const char * dbhome,
 		/*@null@*/ const char * dbfile,
 		/*@unused@*/ /*@null@*/ const char * dbsubfile)
-	/*@modifies dbi, fileSystem @*/
+	/*@globals fileSystem @*/
+	/*@modifies fileSystem @*/
 {
     rpmdb rpmdb = dbi->dbi_rpmdb;
-    DB_ENV * dbenv = dbi->dbi_dbenv;
+    DB_ENV * dbenv = rpmdb->db_dbenv;
     int rc;
 
-    if (dbenv == NULL) {
-	dbi->dbi_dbenv = NULL;
+    if (dbenv == NULL)
 	return 0;
-    }
 
     rc = dbenv->close(dbenv, 0);
     rc = cvtdberr(dbi, "dbenv->close", rc, _debug);
@@ -115,9 +132,11 @@ static int db_fini(dbiIndex dbi, const char * dbhome,
     if (rpmdb->db_remove_env || dbi->dbi_tear_down) {
 	int xx;
 
+	/*@-moduncon@*/ /* FIX: annotate db3 methods */
 	xx = db_env_create(&dbenv, 0);
+	/*@=moduncon@*/
 	xx = cvtdberr(dbi, "db_env_create", rc, _debug);
-#if DB_VERSION_MAJOR == 3 && DB_VERSION_MINOR != 0
+#if (DB_VERSION_MAJOR == 3 && DB_VERSION_MINOR != 0) || (DB_VERSION_MAJOR == 4)
 	xx = dbenv->remove(dbenv, dbhome, 0);
 #else
 	xx = dbenv->remove(dbenv, dbhome, NULL, 0);
@@ -129,7 +148,6 @@ static int db_fini(dbiIndex dbi, const char * dbhome,
 			dbhome, dbfile);
 
     }
-    dbi->dbi_dbenv = NULL;
     return rc;
 }
 
@@ -139,10 +157,13 @@ static int db3_fsync_disable(/*@unused@*/ int fd)
     return 0;
 }
 
+/*@-moduncon@*/ /* FIX: annotate db3 methods */
 static int db_init(dbiIndex dbi, const char * dbhome,
 		/*@null@*/ const char * dbfile,
 		/*@unused@*/ /*@null@*/ const char * dbsubfile,
 		/*@out@*/ DB_ENV ** dbenvp)
+	/*@globals rpmGlobalMacroContext,
+		fileSystem @*/
 	/*@modifies dbi, *dbenvp, fileSystem @*/
 {
     rpmdb rpmdb = dbi->dbi_rpmdb;
@@ -188,17 +209,19 @@ static int db_init(dbiIndex dbi, const char * dbhome,
 	return 1;
 
   { int xx;
+    /*@-noeffectuncon@*/ /* FIX: annotate db3 methods */
     dbenv->set_errcall(dbenv, rpmdb->db_errcall);
     dbenv->set_errfile(dbenv, rpmdb->db_errfile);
     dbenv->set_errpfx(dbenv, rpmdb->db_errpfx);
  /* dbenv->set_paniccall(???) */
-    (void) dbenv->set_verbose(dbenv, DB_VERB_CHKPOINT,
+    /*@=noeffectuncon@*/
+    xx = dbenv->set_verbose(dbenv, DB_VERB_CHKPOINT,
 		(dbi->dbi_verbose & DB_VERB_CHKPOINT));
-    (void) dbenv->set_verbose(dbenv, DB_VERB_DEADLOCK,
+    xx = dbenv->set_verbose(dbenv, DB_VERB_DEADLOCK,
 		(dbi->dbi_verbose & DB_VERB_DEADLOCK));
-    (void) dbenv->set_verbose(dbenv, DB_VERB_RECOVERY,
+    xx = dbenv->set_verbose(dbenv, DB_VERB_RECOVERY,
 		(dbi->dbi_verbose & DB_VERB_RECOVERY));
-    (void) dbenv->set_verbose(dbenv, DB_VERB_WAITSFOR,
+    xx = dbenv->set_verbose(dbenv, DB_VERB_WAITSFOR,
 		(dbi->dbi_verbose & DB_VERB_WAITSFOR));
  /* dbenv->set_lg_max(???) */
  /* dbenv->set_lk_conflicts(???) */
@@ -211,7 +234,7 @@ static int db_init(dbiIndex dbi, const char * dbhome,
  /* dbenv->set_tx_max(???) */
  /* dbenv->set_tx_recover(???) */
     if (dbi->dbi_no_fsync) {
-#if DB_VERSION_MAJOR == 3 && DB_VERSION_MINOR != 0
+#if (DB_VERSION_MAJOR == 3 && DB_VERSION_MINOR != 0) || (DB_VERSION_MAJOR == 4)
 	xx = db_env_set_func_fsync(db3_fsync_disable);
 #else
 	xx = dbenv->set_func_fsync(dbenv, db3_fsync_disable);
@@ -220,7 +243,7 @@ static int db_init(dbiIndex dbi, const char * dbhome,
     }
 
 /* XXX 3.3.4 change. */
-#if DB_VERSION_MAJOR == 3 && DB_VERSION_MINOR == 3
+#if (DB_VERSION_MAJOR == 3 && DB_VERSION_MINOR == 3) || (DB_VERSION_MAJOR == 4)
     if ((dbi->dbi_ecflags & DB_CLIENT) && dbi->dbi_host) {
 	xx = dbenv->set_rpc_server(dbenv, NULL, dbi->dbi_host,
 		dbi->dbi_cl_timeout, dbi->dbi_sv_timeout, 0);
@@ -244,14 +267,16 @@ static int db_init(dbiIndex dbi, const char * dbhome,
 	root = (dbi->dbi_root ? dbi->dbi_root : rpmdb->db_root);
 	if ((root[0] == '/' && root[1] == '\0') || rpmdb->db_chrootDone)
 	    root = NULL;
+	/*@-mods@*/
 	tmpdir = rpmGenPath(root, dbi->dbi_tmpdir, NULL);
+	/*@=mods@*/
 	xx = dbenv->set_tmp_dir(dbenv, tmpdir);
 	xx = cvtdberr(dbi, "dbenv->set_tmp_dir", rc, _debug);
 	tmpdir = _free(tmpdir);
     }
   }
 
-#if DB_VERSION_MAJOR == 3 && DB_VERSION_MINOR != 0
+#if (DB_VERSION_MAJOR == 3 && DB_VERSION_MINOR != 0) || (DB_VERSION_MAJOR == 4)
     rc = dbenv->open(dbenv, dbhome, eflags, dbi->dbi_perms);
 #else
     rc = dbenv->open(dbenv, dbhome, NULL, eflags, dbi->dbi_perms);
@@ -272,8 +297,10 @@ errxit:
     }
     return rc;
 }
+/*@=moduncon@*/
 
 static int db3sync(dbiIndex dbi, unsigned int flags)
+	/*@globals fileSystem @*/
 	/*@modifies fileSystem @*/
 {
     DB * db = dbi->dbi_db;
@@ -289,6 +316,7 @@ static int db3sync(dbiIndex dbi, unsigned int flags)
 }
 
 static int db3c_del(dbiIndex dbi, DBC * dbcursor, u_int32_t flags)
+	/*@globals fileSystem @*/
 	/*@modifies fileSystem @*/
 {
     int rc;
@@ -300,6 +328,7 @@ static int db3c_del(dbiIndex dbi, DBC * dbcursor, u_int32_t flags)
 
 /*@unused@*/ static int db3c_dup(dbiIndex dbi, DBC * dbcursor, DBC ** dbcp,
 		u_int32_t flags)
+	/*@globals fileSystem @*/
 	/*@modifies *dbcp, fileSystem @*/
 {
     int rc;
@@ -307,11 +336,14 @@ static int db3c_del(dbiIndex dbi, DBC * dbcursor, u_int32_t flags)
     if (dbcp) *dbcp = NULL;
     rc = dbcursor->c_dup(dbcursor, dbcp, flags);
     rc = cvtdberr(dbi, "dbcursor->c_dup", rc, _debug);
+    /*@-nullstate @*/ /* FIX: *dbcp can be NULL */
     return rc;
+    /*@=nullstate @*/
 }
 
 static int db3c_get(dbiIndex dbi, DBC * dbcursor,
 		DBT * key, DBT * data, u_int32_t flags)
+	/*@globals fileSystem @*/
 	/*@modifies fileSystem @*/
 {
     int _printit;
@@ -335,6 +367,7 @@ static int db3c_get(dbiIndex dbi, DBC * dbcursor,
 
 static int db3c_put(dbiIndex dbi, DBC * dbcursor,
 		DBT * key, DBT * data, u_int32_t flags)
+	/*@globals fileSystem @*/
 	/*@modifies fileSystem @*/
 {
     int rc;
@@ -346,6 +379,7 @@ static int db3c_put(dbiIndex dbi, DBC * dbcursor,
 }
 
 static inline int db3c_close(dbiIndex dbi, /*@only@*/ /*@null@*/ DBC * dbcursor)
+	/*@globals fileSystem @*/
 	/*@modifies fileSystem @*/
 {
     int rc;
@@ -359,6 +393,7 @@ static inline int db3c_close(dbiIndex dbi, /*@only@*/ /*@null@*/ DBC * dbcursor)
 
 static inline int db3c_open(dbiIndex dbi, /*@null@*/ /*@out@*/ DBC ** dbcp,
 		int dbiflags)
+	/*@globals fileSystem @*/
 	/*@modifies *dbcp, fileSystem @*/
 {
     DB * db = dbi->dbi_db;
@@ -382,6 +417,7 @@ static inline int db3c_open(dbiIndex dbi, /*@null@*/ /*@out@*/ DBC ** dbcp,
 
 static int db3cclose(dbiIndex dbi, /*@only@*/ /*@null@*/ DBC * dbcursor,
 		unsigned int flags)
+	/*@globals fileSystem @*/
 	/*@modifies dbi, fileSystem @*/
 {
     int rc = 0;
@@ -393,11 +429,15 @@ static int db3cclose(dbiIndex dbi, /*@only@*/ /*@null@*/ DBC * dbcursor,
     if (!dbi->dbi_use_cursors)
 	return 0;
 
+    /*@-branchstate@*/
     if (dbcursor == NULL)
 	dbcursor = dbi->dbi_rmw;
+    /*@=branchstate@*/
     if (dbcursor) {
+	/*@-branchstate@*/
 	if (dbcursor == dbi->dbi_rmw)
 	    dbi->dbi_rmw = NULL;
+	/*@=branchstate@*/
 	rc = db3c_close(dbi, dbcursor);
     }
     /*@-usereleased -compdef@*/ return rc; /*@=usereleased =compdef@*/
@@ -405,6 +445,7 @@ static int db3cclose(dbiIndex dbi, /*@only@*/ /*@null@*/ DBC * dbcursor,
 
 static int db3copen(dbiIndex dbi,
 		/*@null@*/ /*@out@*/ DBC ** dbcp, unsigned int flags)
+	/*@globals fileSystem @*/
 	/*@modifies dbi, *dbcp, fileSystem @*/
 {
     DBC * dbcursor;
@@ -434,6 +475,7 @@ static int db3cput(dbiIndex dbi, DBC * dbcursor,
 		const void * keyp, size_t keylen,
 		const void * datap, size_t datalen,
 		/*@unused@*/ unsigned int flags)
+	/*@globals fileSystem @*/
 	/*@modifies fileSystem @*/
 {
     DB * db = dbi->dbi_db;
@@ -464,6 +506,7 @@ static int db3cput(dbiIndex dbi, DBC * dbcursor,
 static int db3cdel(dbiIndex dbi, DBC * dbcursor,
 		const void * keyp, size_t keylen,
 		/*@unused@*/ unsigned int flags)
+	/*@globals fileSystem @*/
 	/*@modifies fileSystem @*/
 {
     DB * db = dbi->dbi_db;
@@ -499,6 +542,7 @@ static int db3cget(dbiIndex dbi, DBC * dbcursor,
 		/*@null@*/ void ** keyp, /*@null@*/ size_t * keylen,
 		/*@null@*/ void ** datap, /*@null@*/ size_t * datalen,
 		/*@unused@*/ unsigned int flags)
+	/*@globals fileSystem @*/
 	/*@modifies *keyp, *keylen, *datap, *datalen, fileSystem @*/
 {
     DB * db = dbi->dbi_db;
@@ -549,6 +593,7 @@ static int db3cget(dbiIndex dbi, DBC * dbcursor,
 static int db3ccount(dbiIndex dbi, DBC * dbcursor,
 		/*@null@*/ /*@out@*/ unsigned int * countp,
 		/*@unused@*/ unsigned int flags)
+	/*@globals fileSystem @*/
 	/*@modifies *countp, fileSystem @*/
 {
     db_recno_t count = 0;
@@ -569,7 +614,8 @@ static int db3byteswapped(dbiIndex dbi)	/*@*/
     int rc = 0;
 
     if (db != NULL) {
-#if DB_VERSION_MAJOR == 3 && DB_VERSION_MINOR == 3 && DB_VERSION_PATCH == 11
+#if (DB_VERSION_MAJOR == 3 && DB_VERSION_MINOR == 3 && DB_VERSION_PATCH == 11) \
+ || (DB_VERSION_MAJOR == 4)
 	int isswapped = 0;
 	rc = db->get_byteswapped(db, &isswapped);
 	if (rc == 0)
@@ -583,6 +629,7 @@ static int db3byteswapped(dbiIndex dbi)	/*@*/
 }
 
 static int db3stat(dbiIndex dbi, unsigned int flags)
+	/*@globals fileSystem @*/
 	/*@modifies dbi, fileSystem @*/
 {
     DB * db = dbi->dbi_db;
@@ -597,7 +644,7 @@ static int db3stat(dbiIndex dbi, unsigned int flags)
 	flags = 0;
     dbi->dbi_stats = _free(dbi->dbi_stats);
 /* XXX 3.3.4 change. */
-#if DB_VERSION_MAJOR == 3 && DB_VERSION_MINOR == 3
+#if (DB_VERSION_MAJOR == 3 && DB_VERSION_MINOR == 3) || (DB_VERSION_MAJOR == 4)
     rc = db->stat(db, &dbi->dbi_stats, flags);
 #else
     rc = db->stat(db, &dbi->dbi_stats, NULL, flags);
@@ -606,7 +653,10 @@ static int db3stat(dbiIndex dbi, unsigned int flags)
     return rc;
 }
 
+/*@-moduncon@*/ /* FIX: annotate db3 methods */
 static int db3close(/*@only@*/ dbiIndex dbi, /*@unused@*/ unsigned int flags)
+	/*@globals rpmGlobalMacroContext,
+		fileSystem @*/
 	/*@modifies dbi, fileSystem @*/
 {
     rpmdb rpmdb = dbi->dbi_rpmdb;
@@ -633,13 +683,15 @@ static int db3close(/*@only@*/ dbiIndex dbi, /*@unused@*/ unsigned int flags)
      * Either the root or directory components may be a URL. Concatenate,
      * convert the URL to a path, and add the name of the file.
      */
+    /*@-mods@*/
     urlfn = rpmGenPath(root, home, NULL);
+    /*@=mods@*/
     (void) urlPath(urlfn, &dbhome);
     if (dbi->dbi_temporary) {
 	dbfile = NULL;
 	dbsubfile = NULL;
     } else {
-#ifdef	HACK
+#ifdef	HACK	/* XXX necessary to support dbsubfile */
 	dbfile = (dbi->dbi_file ? dbi->dbi_file : db3basename);
 	dbsubfile = (dbi->dbi_subfile ? dbi->dbi_subfile : tagName(dbi->dbi_rpmtag));
 #else
@@ -661,36 +713,44 @@ static int db3close(/*@only@*/ dbiIndex dbi, /*@unused@*/ unsigned int flags)
 
     }
 
-    dbi->dbi_dbinfo = _free(dbi->dbi_dbinfo);
-
-    if (dbi->dbi_use_dbenv) {
-	/*@-nullstate@*/
-	xx = db_fini(dbi, (dbhome ? dbhome : ""), dbfile, dbsubfile);
-	/*@=nullstate@*/
+    if (rpmdb->db_dbenv != NULL && dbi->dbi_use_dbenv) {
+	if (rpmdb->db_opens == 1) {
+	    /*@-nullstate@*/
+	    xx = db_fini(dbi, (dbhome ? dbhome : ""), dbfile, dbsubfile);
+	    /*@=nullstate@*/
+	    rpmdb->db_dbenv = NULL;
+	}
+	rpmdb->db_opens--;
     }
 
     if (dbi->dbi_verify_on_close && !dbi->dbi_temporary) {
 	DB_ENV * dbenv = NULL;
 
+	/*@-moduncon@*/ /* FIX: annotate db3 methods */
 	rc = db_env_create(&dbenv, 0);
+	/*@=moduncon@*/
 	rc = cvtdberr(dbi, "db_env_create", rc, _debug);
 	if (rc || dbenv == NULL) goto exit;
 
+	/*@-noeffectuncon@*/ /* FIX: annotate db3 methods */
 	dbenv->set_errcall(dbenv, rpmdb->db_errcall);
 	dbenv->set_errfile(dbenv, rpmdb->db_errfile);
 	dbenv->set_errpfx(dbenv, rpmdb->db_errpfx);
  /*	dbenv->set_paniccall(???) */
-	(void) dbenv->set_verbose(dbenv, DB_VERB_CHKPOINT,
+	/*@=noeffectuncon@*/
+	xx = dbenv->set_verbose(dbenv, DB_VERB_CHKPOINT,
 		(dbi->dbi_verbose & DB_VERB_CHKPOINT));
-	(void) dbenv->set_verbose(dbenv, DB_VERB_DEADLOCK,
+	xx = dbenv->set_verbose(dbenv, DB_VERB_DEADLOCK,
 		(dbi->dbi_verbose & DB_VERB_DEADLOCK));
-	(void) dbenv->set_verbose(dbenv, DB_VERB_RECOVERY,
+	xx = dbenv->set_verbose(dbenv, DB_VERB_RECOVERY,
 		(dbi->dbi_verbose & DB_VERB_RECOVERY));
-	(void) dbenv->set_verbose(dbenv, DB_VERB_WAITSFOR,
+	xx = dbenv->set_verbose(dbenv, DB_VERB_WAITSFOR,
 		(dbi->dbi_verbose & DB_VERB_WAITSFOR));
 
 	if (dbi->dbi_tmpdir) {
+	    /*@-mods@*/
 	    const char * tmpdir = rpmGenPath(root, dbi->dbi_tmpdir, NULL);
+	    /*@=mods@*/
 	    rc = dbenv->set_tmp_dir(dbenv, tmpdir);
 	    rc = cvtdberr(dbi, "dbenv->set_tmp_dir", rc, _debug);
 	    tmpdir = _free(tmpdir);
@@ -702,11 +762,15 @@ static int db3close(/*@only@*/ dbiIndex dbi, /*@unused@*/ unsigned int flags)
 	rc = cvtdberr(dbi, "dbenv->open", rc, _debug);
 	if (rc) goto exit;
 
+	/*@-moduncon@*/ /* FIX: annotate db3 methods */
 	rc = db_create(&db, dbenv, 0);
+	/*@=moduncon@*/
 	rc = cvtdberr(dbi, "db_create", rc, _debug);
 
 	if (db != NULL) {
+		/*@-mods@*/
 		const char * dbf = rpmGetPath(dbhome, "/", dbfile, NULL);
+		/*@=mods@*/
 
 		rc = db->verify(db, dbf, NULL, NULL, flags);
 		rc = cvtdberr(dbi, "db->verify", rc, _debug);
@@ -736,8 +800,11 @@ exit:
 
     return rc;
 }
+/*@=moduncon@*/
 
 static int db3open(/*@keep@*/ rpmdb rpmdb, int rpmtag, dbiIndex * dbip)
+	/*@globals rpmGlobalMacroContext,
+		fileSystem @*/
 	/*@modifies *dbip, fileSystem @*/
 {
     /*@-nestedextern@*/
@@ -765,10 +832,12 @@ static int db3open(/*@keep@*/ rpmdb rpmdb, int rpmtag, dbiIndex * dbip)
     /*
      * Parse db configuration parameters.
      */
+    /*@-mods@*/
     if ((dbi = db3New(rpmdb, rpmtag)) == NULL)
 	/*@-nullstate@*/
 	return 1;
 	/*@=nullstate@*/
+    /*@=mods@*/
     dbi->dbi_api = DB_VERSION_MAJOR;
 
     /*
@@ -783,13 +852,15 @@ static int db3open(/*@keep@*/ rpmdb rpmdb, int rpmtag, dbiIndex * dbip)
      * Either the root or directory components may be a URL. Concatenate,
      * convert the URL to a path, and add the name of the file.
      */
+    /*@-mods@*/
     urlfn = rpmGenPath(root, home, NULL);
+    /*@=mods@*/
     (void) urlPath(urlfn, &dbhome);
     if (dbi->dbi_temporary) {
 	dbfile = NULL;
 	dbsubfile = NULL;
     } else {
-#ifdef	HACK
+#ifdef	HACK	/* XXX necessary to support dbsubfile */
 	dbfile = (dbi->dbi_file ? dbi->dbi_file : db3basename);
 	dbsubfile = (dbi->dbi_subfile ? dbi->dbi_subfile : tagName(dbi->dbi_rpmtag));
 #else
@@ -857,7 +928,9 @@ static int db3open(/*@keep@*/ rpmdb rpmdb, int rpmtag, dbiIndex * dbip)
 	    }
 
 	} else {	/* dbhome is writable, check for persistent dbenv. */
+	    /*@-mods@*/
 	    const char * dbf = rpmGetPath(dbhome, "/__db.001", NULL);
+	    /*@=mods@*/
 
 	    if (access(dbf, F_OK) == -1) {
 		/* ... non-existent (or unwritable) DBENV, will create ... */
@@ -883,7 +956,9 @@ static int db3open(/*@keep@*/ rpmdb rpmdb, int rpmtag, dbiIndex * dbip)
     if ((oflags & DB_CREATE) && (oflags & DB_RDONLY)) {
 	/* dbhome is writable, and DB->open flags may conflict. */
 	const char * dbfn = (dbfile ? dbfile : tagName(dbi->dbi_rpmtag));
+	/*@-mods@*/
 	const char * dbf = rpmGetPath(dbhome, "/", dbfn, NULL);
+	/*@=mods@*/
 
 	if (access(dbf, F_OK) == -1) {
 	    /* File does not exist, DB->open might create ... */
@@ -908,10 +983,20 @@ static int db3open(/*@keep@*/ rpmdb rpmdb, int rpmtag, dbiIndex * dbip)
     if (oflags & DB_RDONLY)
 	dbi->dbi_verify_on_close = 0;
 
-    dbi->dbi_dbinfo = NULL;
-
-    if (dbi->dbi_use_dbenv)
-	rc = db_init(dbi, dbhome, dbfile, dbsubfile, &dbenv);
+    if (dbi->dbi_use_dbenv) {
+	/*@-mods@*/
+	if (rpmdb->db_dbenv == NULL) {
+	    rc = db_init(dbi, dbhome, dbfile, dbsubfile, &dbenv);
+	    if (rc == 0) {
+		rpmdb->db_dbenv = dbenv;
+		rpmdb->db_opens = 1;
+	    }
+	} else {
+	    dbenv = rpmdb->db_dbenv;
+	    rpmdb->db_opens++;
+	}
+	/*@=mods@*/
+    }
 
     rpmMessage(RPMMESS_DEBUG, _("opening  db index       %s/%s %s mode=0x%x\n"),
 		dbhome, (dbfile ? dbfile : tagName(dbi->dbi_rpmtag)),
@@ -920,7 +1005,9 @@ static int db3open(/*@keep@*/ rpmdb rpmdb, int rpmtag, dbiIndex * dbip)
     if (rc == 0) {
 	static int _lockdbfd = 0;
 
+	/*@-moduncon@*/ /* FIX: annotate db3 methods */
 	rc = db_create(&db, dbenv, dbi->dbi_cflags);
+	/*@=moduncon@*/
 	rc = cvtdberr(dbi, "db_create", rc, _debug);
 	if (rc == 0 && db != NULL) {
 	    if (rc == 0 && dbi->dbi_lorder) {
@@ -936,7 +1023,7 @@ static int db3open(/*@keep@*/ rpmdb rpmdb, int rpmtag, dbiIndex * dbip)
 		rc = cvtdberr(dbi, "db->set_pagesize", rc, _debug);
 	    }
 /* XXX 3.3.4 change. */
-#if DB_VERSION_MAJOR == 3 && DB_VERSION_MINOR == 3
+#if (DB_VERSION_MAJOR == 3 && DB_VERSION_MINOR == 3) || (DB_VERSION_MAJOR == 4)
 	    if (rc == 0 &&
 			rpmdb->db_malloc && rpmdb->db_realloc && rpmdb->db_free)
 	    {
@@ -970,7 +1057,7 @@ static int db3open(/*@keep@*/ rpmdb rpmdb, int rpmtag, dbiIndex * dbip)
 			if (rc) break;
 		    }
 /* XXX db-3.2.9 has added a DB arg to the call. */
-#if DB_VERSION_MAJOR == 3 && DB_VERSION_MINOR > 2
+#if (DB_VERSION_MAJOR == 3 && DB_VERSION_MINOR > 2) || (DB_VERSION_MAJOR == 4)
 		    if (dbi->dbi_h_hash_fcn) {
 			rc = db->set_h_hash(db, dbi->dbi_h_hash_fcn);
 			rc = cvtdberr(dbi, "db->set_h_hash", rc, _debug);
@@ -995,7 +1082,7 @@ static int db3open(/*@keep@*/ rpmdb rpmdb, int rpmtag, dbiIndex * dbip)
 			if (rc) break;
 		    }
 /* XXX db-3.2.9 has added a DB arg to the call. */
-#if DB_VERSION_MAJOR == 3 && DB_VERSION_MINOR > 2
+#if (DB_VERSION_MAJOR == 3 && DB_VERSION_MINOR > 2) || (DB_VERSION_MAJOR == 4)
 		    if (dbi->dbi_bt_compare_fcn) {
 			rc = db->set_bt_compare(db, dbi->dbi_bt_compare_fcn);
 			rc = cvtdberr(dbi, "db->set_bt_compare", rc, _debug);
@@ -1044,7 +1131,6 @@ static int db3open(/*@keep@*/ rpmdb rpmdb, int rpmtag, dbiIndex * dbip)
 		    break;
 		}
 	    }
-	    dbi->dbi_dbinfo = NULL;
 
 	    if (rc == 0) {
 		const char * dbfullpath;
@@ -1059,14 +1145,20 @@ static int db3open(/*@keep@*/ rpmdb rpmdb, int rpmtag, dbiIndex * dbip)
 		t = stpcpy(t, dbhome);
 		if (dbfile)
 		    t = stpcpy( stpcpy( t, "/"), dbfile);
+#ifdef	HACK	/* XXX necessary to support dbsubfile */
 		dbpath = (!dbi->dbi_use_dbenv && !dbi->dbi_temporary)
 			? dbfullpath : dbfile;
+#else
+		dbpath = (!dbi->dbi_temporary)
+			? dbfullpath : dbfile;
+#endif
 
 		rc = db->open(db, dbpath, dbsubfile,
 		    dbi->dbi_type, oflags, dbi->dbi_perms);
 
 		if (rc == 0 && dbi->dbi_type == DB_UNKNOWN) {
-#if DB_VERSION_MAJOR == 3 && DB_VERSION_MINOR == 3 && DB_VERSION_PATCH == 11
+#if (DB_VERSION_MAJOR == 3 && DB_VERSION_MINOR == 3 && DB_VERSION_PATCH == 11) \
+ || (DB_VERSION_MAJOR == 4)
 		    DBTYPE dbi_type = DB_UNKNOWN;
 		    xx = db->get_type(db, &dbi_type);
 		    if (xx == 0)
@@ -1146,7 +1238,6 @@ static int db3open(/*@keep@*/ rpmdb rpmdb, int rpmtag, dbiIndex * dbip)
     }
 
     dbi->dbi_db = db;
-    dbi->dbi_dbenv = dbenv;
 
     if (rc == 0 && dbi->dbi_db != NULL && dbip != NULL) {
 	dbi->dbi_vec = &db3vec;
@@ -1158,17 +1249,19 @@ static int db3open(/*@keep@*/ rpmdb rpmdb, int rpmtag, dbiIndex * dbip)
 
     urlfn = _free(urlfn);
 
-    /*@-nullstate@*/
+    /*@-nullstate -compmempass@*/
     return rc;
-    /*@=nullstate@*/
+    /*@=nullstate =compmempass@*/
 }
 
 /** \ingroup db3
  */
 /*@-exportheadervar@*/
+/*@observer@*/ /*@unchecked@*/
 struct _dbiVec db3vec = {
     DB_VERSION_MAJOR, DB_VERSION_MINOR, DB_VERSION_PATCH,
     db3open, db3close, db3sync, db3copen, db3cclose, db3cdel, db3cget, db3cput,
     db3ccount, db3byteswapped, db3stat
 };
 /*@=exportheadervar@*/
+/*@=type@*/
