@@ -47,8 +47,7 @@ enum modes {
 
     MODE_INSTALL	= (1 <<  1),
     MODE_ERASE		= (1 <<  2),
-    MODE_ROLLBACK	= (1 << 14),
-#define	MODES_IE (MODE_INSTALL | MODE_ERASE | MODE_ROLLBACK)
+#define	MODES_IE (MODE_INSTALL | MODE_ERASE)
 
     MODE_BUILD		= (1 <<  4),
     MODE_REBUILD	= (1 <<  5),
@@ -75,33 +74,39 @@ enum modes {
 #define	MODES_FOR_ROOT		(MODES_BT | MODES_IE | MODES_QV | MODES_DB)
 
 /*@-exportheadervar@*/
+/*@unchecked@*/
 extern int _ftp_debug;
+/*@unchecked@*/
 extern int noLibio;
+/*@unchecked@*/
 extern int _rpmio_debug;
+/*@-redecl@*/
+/*@unchecked@*/
 extern int _url_debug;
+/*@=redecl@*/
 
-/*@-varuse@*/
-/*@observer@*/ extern const char * rpmNAME;
-/*@=varuse@*/
-/*@observer@*/ extern const char * rpmEVR;
-/*@-varuse@*/
-extern int rpmFLAGS;
-/*@=varuse@*/
-
-extern struct MacroContext_s rpmCLIMacroContext;
 /*@=exportheadervar@*/
 
 /* options for all executables */
 
+/*@unchecked@*/
 static int help = 0;
+/*@unchecked@*/
 static int noUsageMsg = 0;
-/*@observer@*/ /*@null@*/ static const char * pipeOutput = NULL;
+/*@unchecked@*/ /*@observer@*/ /*@null@*/
+static const char * pipeOutput = NULL;
+/*@unchecked@*/
 static int quiet = 0;
-/*@observer@*/ /*@null@*/ static const char * rcfile = NULL;
-/*@observer@*/ /*@null@*/ static char * rootdir = "/";
+/*@unchecked@*/ /*@observer@*/ /*@null@*/
+static const char * rcfile = NULL;
+/*@unchecked@*/ /*@observer@*/ /*@null@*/
+static char * rootdir = "/";
+/*@unchecked@*/
 static int showrc = 0;
+/*@unchecked@*/
 static int showVersion = 0;
 
+/*@unchecked@*/
 static struct poptOption rpmAllPoptTable[] = {
  { "version", '\0', 0, &showVersion, 0,
 	N_("print the version of rpm being used"),
@@ -153,6 +158,7 @@ static struct poptOption rpmAllPoptTable[] = {
 };
 
 /* the structure describing the options we take and the defaults */
+/*@unchecked@*/
 static struct poptOption optionsTable[] = {
 
  /* XXX colliding options */
@@ -212,6 +218,7 @@ long _stksize = 64 * 1024L;
 #endif
 
 /*@exits@*/ static void argerror(const char * desc)
+	/*@globals fileSystem @*/
 	/*@modifies fileSystem @*/
 {
     fprintf(stderr, _("%s: %s\n"), __progname, desc);
@@ -219,12 +226,14 @@ long _stksize = 64 * 1024L;
 }
 
 static void printVersion(void)
+	/*@globals rpmEVR, fileSystem @*/
 	/*@modifies fileSystem @*/
 {
     fprintf(stdout, _("RPM version %s\n"), rpmEVR);
 }
 
 static void printBanner(void)
+	/*@globals fileSystem @*/
 	/*@modifies fileSystem @*/
 {
     (void) puts(_("Copyright (C) 1998-2000 - Red Hat, Inc."));
@@ -232,6 +241,7 @@ static void printBanner(void)
 }
 
 static void printUsage(void)
+	/*@globals fileSystem @*/
 	/*@modifies fileSystem @*/
 {
     FILE * fp = stdout;
@@ -459,7 +469,12 @@ int main(int argc, const char ** argv)
 		ka->checksigFlags &= ~CHECKSIG_MD5;
 	    else
 #endif
-		/*@-ifempty@*/ ;
+#ifdef	IAM_RPMEIU
+	    if (bigMode & MODES_IE)
+		ia->transFlags |= RPMTRANS_FLAG_NOMD5;
+	    else
+#endif
+		{};
 	    break;
 #endif	/* IAM_RPMQV || IAM_RPMK */
 
@@ -550,7 +565,7 @@ int main(int argc, const char ** argv)
 	case GETOPT_DEFINEMACRO:
 	    if (optArg) {
 		(void) rpmDefineMacro(NULL, optArg, RMIL_CMDLINE);
-		(void) rpmDefineMacro(&rpmCLIMacroContext, optArg,RMIL_CMDLINE);
+		(void) rpmDefineMacro(rpmCLIMacroContext, optArg,RMIL_CMDLINE);
 	    }
 	    noUsageMsg = 1;
 	    break;
@@ -1002,23 +1017,25 @@ int main(int argc, const char ** argv)
 
 #ifdef	IAM_RPMEIU
     case MODE_ERASE:
-	if (!poptPeekArg(optCon))
-	    argerror(_("no packages given for uninstall"));
-
+	ia->rootdir = rootdir;
 	if (ia->noDeps) ia->eraseInterfaceFlags |= UNINSTALL_NODEPS;
 
-	ec = rpmErase(rootdir, (const char **)poptGetArgs(optCon), 
+	if (!poptPeekArg(optCon)) {
+	    if (ia->rbtid == 0)
+		argerror(_("no packages given for erase"));
+ia->transFlags |= RPMTRANS_FLAG_NOMD5;
+ia->probFilter |= RPMPROB_FILTER_OLDPACKAGE;
+	    ec += rpmRollback(ia, NULL);
+	} else {
+	    ec += rpmErase(rootdir, (const char **)poptGetArgs(optCon), 
 			 ia->transFlags, ia->eraseInterfaceFlags);
+	}
 	break;
 
     case MODE_INSTALL:
-
-	if (!poptPeekArg(optCon))
-	    argerror(_("no packages given for install"));
-
 	/* RPMTRANS_FLAG_BUILD_PROBS */
 	/* RPMTRANS_FLAG_KEEPOBSOLETE */
-
+	ia->rootdir = rootdir;
 	if (!ia->incldocs) {
 	    if (ia->transFlags & RPMTRANS_FLAG_NODOCS)
 		;
@@ -1041,18 +1058,21 @@ int main(int argc, const char ** argv)
 	    ia->relocations[ia->numRelocations].newPath = NULL;
 	}
 
-	/*@-compdef@*/ /* FIX: ia->relocations[0].newPath undefined */
-	ec += rpmInstall(rootdir, (const char **)poptGetArgs(optCon), 
-			ia->transFlags, ia->installInterfaceFlags, ia->probFilter,
-			ia->relocations);
+	if (!poptPeekArg(optCon)) {
+	    if (ia->rbtid == 0)
+		argerror(_("no packages given for install"));
+ia->transFlags |= RPMTRANS_FLAG_NOMD5;
+ia->probFilter |= RPMPROB_FILTER_OLDPACKAGE;
+	    ec += rpmRollback(ia, NULL);
+	} else {
+
+	    /*@-compdef@*/ /* FIX: ia->relocations[0].newPath undefined */
+	    ec += rpmInstall(rootdir, (const char **)poptGetArgs(optCon), 
+			ia->transFlags, ia->installInterfaceFlags,
+			ia->probFilter, ia->relocations);
+	}
 	/*@=compdef@*/
 	break;
-
-    case MODE_ROLLBACK:
-	ia->rootdir = rootdir;
-	ec += rpmRollback(ia, (const char **)poptGetArgs(optCon));
-	break;
-
 #endif	/* IAM_RPMEIU */
 
 #ifdef	IAM_RPMQV
@@ -1149,7 +1169,6 @@ int main(int argc, const char ** argv)
 #if !defined(IAM_RPMEIU)
     case MODE_INSTALL:
     case MODE_ERASE:
-    case MODE_ROLLBACK:
 #endif
     case MODE_UNKNOWN:
 	if (!showVersion && !help && !noUsageMsg) printUsage();
@@ -1161,7 +1180,7 @@ exit:
 #endif	/* IAM_RPMBT || IAM_RPMK */
     optCon = poptFreeContext(optCon);
     rpmFreeMacros(NULL);
-    rpmFreeMacros(&rpmCLIMacroContext);
+    rpmFreeMacros(rpmCLIMacroContext);
     rpmFreeRpmrc();
 
     if (pipeChild) {
