@@ -21,6 +21,13 @@
    of _POSIX_VERSION.  */
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
+#if defined(__LCLINT__)
+/*@-superuser -declundef -incondefs @*/	/* LCL: modifies clause missing */
+extern int chroot (const char *__path)
+	/*@globals errno, systemState @*/
+	/*@modifies errno, systemState @*/;
+/*@=superuser =declundef =incondefs @*/
+#endif
 #endif
 
 #if TIME_WITH_SYS_TIME
@@ -38,18 +45,6 @@
 extern time_t timezone;
 #endif
 
-#ifdef	__LCLINT__
-typedef	unsigned int u_int32_t;
-typedef	unsigned short u_int16_t;
-typedef	unsigned char u_int8_t;
-/*@-incondefs@*/	/* LCLint 3.0.0.15 */
-typedef	int int32_t;
-/*@=incondefs@*/
-/*@-declundef@*/
-/*@only@*/ void * alloca (size_t size)	/*@modifies internalState @*/;
-/*@=declundef@*/
-#endif
-
 /* Since major is a function on SVR4, we can't use `ifndef major'.  */
 #if MAJOR_IN_MKDEV
 #include <sys/mkdev.h>
@@ -74,9 +69,6 @@ typedef	int int32_t;
 #include <utime.h>
 #endif
 
-/* Don't use bcopy!  Use memmove if source and destination may overlap,
-   memcpy otherwise.  */
-
 #ifdef HAVE_STRING_H
 # if !STDC_HEADERS && HAVE_MEMORY_H
 #  include <memory.h>
@@ -87,16 +79,12 @@ typedef	int int32_t;
 char *memchr ();
 #endif
 
-#if !defined(HAVE_STPCPY) || defined(__LCLINT__)
-/*@-declundef@*/
-char * stpcpy(char * dest, const char * src)	/*@modifies *dest @*/;
-/*@=declundef@*/
+#if !defined(HAVE_STPCPY)
+char * stpcpy(/*@out@*/ char * dest, const char * src);
 #endif
 
-#if !defined(HAVE_STPNCPY) || defined(__LCLINT__)
-/*@-declundef@*/
-char * stpncpy(char * dest, const char * src, size_t n)	/*@modifies *dest @*/;
-/*@=declundef@*/
+#if !defined(HAVE_STPNCPY)
+char * stpncpy(/*@out@*/ char * dest, const char * src, size_t n);
 #endif
 
 #include <errno.h>
@@ -107,15 +95,26 @@ extern int errno;
 #endif
 
 #ifdef STDC_HEADERS
-/*@-macrounrecog -incondefs -globuse -mustmod@*/	/* FIX: shrug */
+/*@-macrounrecog -incondefs -globuse -mustmod @*/ /* FIX: shrug */
 #define getopt system_getopt
-/*@=macrounrecog =incondefs =globuse =mustmod@*/
+/*@=macrounrecog =incondefs =globuse =mustmod @*/
 /*@-skipansiheaders@*/
 #include <stdlib.h>
 /*@=skipansiheaders@*/
 #undef getopt
+#if defined(__LCLINT__)
+/*@-declundef -incondefs @*/	/* LCL: modifies clause missing */
+extern char * realpath (const char * file_name, /*@out@*/ char * resolved_name)
+	/*@globals errno, fileSystem @*/
+	/*@requires maxSet(resolved_name) >=  (PATH_MAX - 1); @*/
+	/*@modifies *resolved_name, errno, fileSystem @*/;
+/*@=declundef =incondefs @*/
+#endif
 #else /* not STDC_HEADERS */
 char *getenv (const char *name);
+#if ! HAVE_REALPATH
+char *realpath(const char *path, char resolved_path []);
+#endif
 #endif /* STDC_HEADERS */
 
 /* XXX solaris2.5.1 has not */
@@ -158,6 +157,14 @@ char *getenv (const char *name);
 # endif /* HAVE_NDIR_H */
 #endif /* HAVE_DIRENT_H */
 
+#if defined(__LCLINT__)
+/*@-declundef -incondefs @*/ /* LCL: missing annotation */
+/*@only@*/ void * alloca (size_t __size)
+	/*@ensures MaxSet(result) == (__size - 1) @*/
+	/*@*/;
+/*@=declundef =incondefs @*/
+#endif
+
 #ifdef __GNUC__
 # undef alloca
 # define alloca __builtin_alloca
@@ -172,9 +179,31 @@ char *alloca ();
 # endif
 #endif
 
+#if defined (__GLIBC__) && defined(__LCLINT__)
+/*@-declundef@*/
+/*@unchecked@*/
+extern __const __int32_t *__ctype_tolower;
+/*@unchecked@*/
+extern __const __int32_t *__ctype_toupper;
+/*@=declundef@*/
 #include <ctype.h>
 
-#if HAVE_SYS_MMAN_H
+/*@-exportlocal@*/
+extern int isalnum(int) __THROW	/*@*/;
+extern int iscntrl(int) __THROW	/*@*/;
+extern int isgraph(int) __THROW	/*@*/;
+extern int islower(int) __THROW	/*@*/;
+extern int ispunct(int) __THROW	/*@*/;
+extern int isxdigit(int) __THROW	/*@*/;
+extern int isascii(int) __THROW	/*@*/;
+extern int toascii(int) __THROW	/*@*/;
+extern int _toupper(int) __THROW	/*@*/;
+extern int _tolower(int) __THROW	/*@*/;
+/*@=exportlocal@*/
+
+#endif
+
+#if HAVE_SYS_MMAN_H && !defined(__LCLINT__)
 #include <sys/mman.h>
 #endif
 
@@ -192,7 +221,9 @@ char *alloca ();
 #endif
 
 #if HAVE_GETOPT_H
+/*@-noparams@*/
 #include <getopt.h>
+/*@=noparams@*/
 #endif
 
 #if HAVE_GRP_H
@@ -207,31 +238,87 @@ char *alloca ();
 #include <err.h>
 #endif
 
-#if HAVE_MALLOC_H
+#if HAVE_MALLOC_H && !defined(__LCLINT__)
 #include <malloc.h>
 #endif
 
-/*@-declundef@*/
-/*@mayexit@*/ /*@only@*/ void * xmalloc (size_t size)			/*@*/;
-/*@mayexit@*/ /*@only@*/ void * xcalloc (size_t nmemb, size_t size)	/*@*/;
-/*@mayexit@*/ /*@only@*/ void * xrealloc (/*@only@*//*@null@*/ void * ptr,
-					size_t size) /*@*/;
-/*@mayexit@*/ /*@only@*/ char * xstrdup (const char *str)		/*@*/;
-/*@=declundef@*/
-/*@unused@*/ /*@exits@*/ /*@only@*/ void * vmefail(size_t size);
+/*@-declundef -incondefs @*/ /* FIX: these are macros */
+/**
+ */
+/*@mayexit@*/ /*@only@*/ /*@out@*/ void * xmalloc (size_t size)
+	/*@globals errno @*/
+	/*@ensures MaxSet(result) == (size - 1) @*/
+	/*@modifies errno @*/;
+
+/**
+ */
+/*@mayexit@*/ /*@only@*/ void * xcalloc (size_t nmemb, size_t size)
+	/*@ensures MaxSet(result) == (nmemb - 1) @*/
+	/*@*/;
+
+/**
+ * @todo Annotate ptr with returned/out.
+ */
+/*@mayexit@*/ /*@only@*/ void * xrealloc (/*@null@*/ /*@only@*/ void * ptr,
+					size_t size)
+	/*@ensures MaxSet(result) == (size - 1) @*/
+	/*@modifies *ptr @*/;
+
+/**
+ */
+/*@mayexit@*/ /*@only@*/ char * xstrdup (const char *str)
+	/*@*/;
+/*@=declundef =incondefs @*/
+
+/**
+ */
+/*@unused@*/ /*@exits@*/ /*@only@*/ void * vmefail(size_t size)
+	/*@*/;
 
 #if HAVE_MCHECK_H
 #include <mcheck.h>
+#if defined(__LCLINT__)
+/*@-declundef -incondefs @*/ /* LCL: missing annotations */
+#if 0
+enum mcheck_status
+  {
+    MCHECK_DISABLED = -1,       /* Consistency checking is not turned on.  */
+    MCHECK_OK,                  /* Block is fine.  */
+    MCHECK_FREE,                /* Block freed twice.  */
+    MCHECK_HEAD,                /* Memory before the block was clobbered.  */
+    MCHECK_TAIL                 /* Memory after the block was clobbered.  */
+  };
 #endif
 
+extern int mcheck (void (*__abortfunc) (enum mcheck_status))
+	/*@globals internalState@*/
+	/*@modifies internalState @*/;
+extern int mcheck_pedantic (void (*__abortfunc) (enum mcheck_status))
+	/*@globals internalState@*/
+	/*@modifies internalState @*/;
+extern void mcheck_check_all (void)
+	/*@globals internalState@*/
+	/*@modifies internalState @*/;
+extern enum mcheck_status mprobe (void *__ptr)
+	/*@globals internalState@*/
+	/*@modifies internalState @*/;
+extern void mtrace (void)
+	/*@globals internalState@*/
+	/*@modifies internalState @*/;
+extern void muntrace (void)
+	/*@globals internalState@*/
+	/*@modifies internalState @*/;
+/*@=declundef =incondefs @*/
+#endif /* defined(__LCLINT__) */
 
 /* Memory allocation via macro defs to get meaningful locations from mtrace() */
-#if HAVE_MCHECK_H && defined(__GNUC__)
+#if defined(__GNUC__)
 #define	xmalloc(_size) 		(malloc(_size) ? : vmefail(_size))
 #define	xcalloc(_nmemb, _size)	(calloc((_nmemb), (_size)) ? : vmefail(_size))
 #define	xrealloc(_ptr, _size)	(realloc((_ptr), (_size)) ? : vmefail(_size))
 #define	xstrdup(_str)	(strcpy((malloc(strlen(_str)+1) ? : vmefail(strlen(_str)+1)), (_str)))
-#endif	/* HAVE_MCHECK_H && defined(__GNUC__) */
+#endif	/* defined(__GNUC__) */
+#endif	/* HAVE_MCHECK_H */
 
 /* Retrofit glibc __progname */
 #if defined __GLIBC__ && __GLIBC__ >= 2
@@ -249,9 +336,7 @@ char *alloca ();
 const char *__progname;
 
 #if HAVE_NETDB_H
-#ifndef __LCLINT__
 #include <netdb.h>
-#endif	/* __LCLINT__ */
 #endif
 
 #if HAVE_PWD_H
@@ -286,37 +371,138 @@ const char *__progname;
 
 #if !defined(USE_GNU_GLOB)
 #if HAVE_FNMATCH_H
+/*@-noparams@*/
 #include <fnmatch.h>
+/*@=noparams@*/
 #endif
 
 #if HAVE_GLOB_H
+/*@-noparams@*/
 #include <glob.h>
+/*@=noparams@*/
 #endif
 #else
+/*@-noparams@*/
 #include "misc/glob.h"
 #include "misc/fnmatch.h"
+/*@=noparams@*/
+#endif
+
+#if defined(__LCLINT__)
+/*@-declundef -incondefs @*/ /* LCL: missing annotation */
+#if 0
+typedef /*@concrete@*/ struct
+  {
+    size_t gl_pathc;
+    char **gl_pathv;
+    size_t gl_offs;
+    int gl_flags;
+
+    void (*gl_closedir) (void *);
+#ifdef _GNU_SOURCE
+    struct dirent *(*gl_readdir) (void *);
+#else
+    void *(*gl_readdir) (void *);
+#endif
+    ptr_t (*gl_opendir) (const char *);
+#ifdef _GNU_SOURCE
+    int (*gl_lstat) (const char *restrict, struct stat *restrict);
+    int (*gl_stat) (const char *restrict, struct stat *restrict);
+#else
+    int (*gl_lstat) (const char *restrict, void *restrict);
+    int (*gl_stat) (const char *restrict, void *restrict);
+#endif
+  } glob_t;
+#endif
+
+#if 0
+/*@-constuse@*/
+/*@constant int GLOB_ERR@*/
+/*@constant int GLOB_MARK@*/
+/*@constant int GLOB_NOSORT@*/
+/*@constant int GLOB_DOOFFS@*/
+/*@constant int GLOB_NOCHECK@*/
+/*@constant int GLOB_APPEND@*/
+/*@constant int GLOB_NOESCAPE@*/
+/*@constant int GLOB_PERIOD@*/
+
+#ifdef _GNU_SOURCE
+/*@constant int GLOB_MAGCHAR@*/
+/*@constant int GLOB_ALTDIRFUNC@*/
+/*@constant int GLOB_BRACE@*/
+/*@constant int GLOB_NOMAGIC@*/
+/*@constant int GLOB_TILDE@*/
+/*@constant int GLOB_ONLYDIR@*/
+/*@constant int GLOB_TILDE_CHECK@*/
+#endif
+
+/*@constant int GLOB_FLAGS@*/
+
+/*@constant int GLOB_NOSPACE@*/
+/*@constant int GLOB_ABORTED@*/
+/*@constant int GLOB_NOMATCH@*/
+/*@constant int GLOB_NOSYS@*/
+#ifdef _GNU_SOURCE
+/*@constant int GLOB_ABEND@*/
+#endif
+/*@=constuse@*/
+#endif
+
+extern int glob (const char *pattern, int flags,
+                      int (*errfunc) (const char *, int),
+                      /*@out@*/ glob_t *pglob)
+	/*@globals errno, fileSystem @*/
+	/*@modifies *pglob, errno, fileSystem @*/;
+	/* XXX only annotation is a white lie */
+extern void globfree (/*@only@*/ glob_t *pglob)
+	/*@modifies *pglob @*/;
+#ifdef _GNU_SOURCE
+extern int glob_pattern_p (const char *pattern, int quote)
+	/*@*/;
+#endif
+
+#if 0
+/*@-constuse@*/
+/*@constant int FNM_PATHNAME@*/
+/*@constant int FNM_NOESCAPE@*/
+/*@constant int FNM_PERIOD@*/
+
+#ifdef _GNU_SOURCE
+/*@constant int FNM_FILE_NAME@*/	/* GNU extension */
+/*@constant int FNM_LEADING_DIR@*/	/* GNU extension */
+/*@constant int FNM_CASEFOLD@*/		/* GNU extension */
+/*@constant int FNM_EXTMATCH@*/		/* GNU extension */
+#endif
+
+/*@constant int FNM_NOMATCH@*/
+
+#ifdef _XOPEN_SOURCE
+/*@constant int FNM_NOSYS@*/		/* X/Open */
+#endif
+/*@=constuse@*/
+#endif
+
+extern int fnmatch (const char *pattern, const char *string, int flags)
+	/*@*/;
+/*@=declundef =incondefs @*/
 #endif
 
 #if ! HAVE_S_IFSOCK
-#define S_IFSOCK (0xC000)
+#define S_IFSOCK (0xc000)
 #endif
 
 #if ! HAVE_S_ISLNK
-#define S_ISLNK(mode) ((mode & 0xF000) == S_IFLNK)
+#define S_ISLNK(mode) ((mode & 0xf000) == S_IFLNK)
 #endif
 
 #if ! HAVE_S_ISSOCK
-#define S_ISSOCK(mode) ((mode & 0xF000) == S_IFSOCK)
+#define S_ISSOCK(mode) ((mode & 0xf000) == S_IFSOCK)
 #endif
 
 #if NEED_STRINGS_H
 #include <strings.h>
 #endif
 
-#if ! HAVE_REALPATH
-char *realpath(const char *path, char resolved_path []);
-#endif
-
 #if NEED_MYREALLOC
 #define realloc(ptr,size) myrealloc(ptr,size)
 extern void *myrealloc(void *, size_t);
@@ -329,12 +515,10 @@ extern void unsetenv(const char *name);
 
 #if HAVE_SYS_SOCKET_H
 #include <sys/types.h>
-#ifndef	__LCLINT__
 #include <sys/socket.h>
-#endif	/* __LCLINT__ */
 #endif
 
-#if HAVE_SYS_SELECT_H
+#if HAVE_SYS_SELECT_H && !defined(__LCLINT__)
 #include <sys/select.h>
 #endif
 
