@@ -22,8 +22,9 @@ int main(int argc, char **argv)
     } else {
 	fdi = Fopen(argv[1], "r.ufdio");
     }
-    if (fdi == NULL || Ferror(fdi)) {
-	perror("input");
+    if (Ferror(fdi)) {
+	fprintf(stderr, "%s: %s: %s\n", argv[0],
+		(argc == 1 ? "<stdin>" : argv[1]), Fstrerror(fdi));
 	exit(EXIT_FAILURE);
     }
 
