@@ -19,19 +19,21 @@ int main(int argc, char **argv)
 	fdi = Fopen(argv[1], "r.ufdio");
     }
     if (Ferror(fdi)) {
-	perror("input");
+	fprintf(stderr, "%s: %s: %s\n", argv[0],
+		(argc == 1 ? "<stdin>" : argv[1]), Fstrerror(fdi));
 	exit(1);
     }
 
     readLead(fdi, &lead);
     rpmReadSignature(fdi, &sig, lead.signature_type);
     switch (lead.signature_type) {
-      case RPMSIG_NONE:
+    case RPMSIGTYPE_NONE:
 	fprintf(stderr, _("No signature available.\n"));
 	break;
-      default:
+    default:
 	fdo = Fopen("-", "w.ufdio");
 	rpmWriteSignature(fdo, sig);
+	break;
     }
     
     return 0;
